"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q, limit=2):
		""" qwant.com search engine

			q 		  : Query for search
			limit	  : Number of pages
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self._pages = ''
		self._json = []
		self._links = []
		self._links_with_title = []
		self.qwant = 'https://api.qwant.com/v3/search/web'

	def run_crawl(self):
		page = 1
		set_page = lambda x: (x-1)*10
		payload = {'q': self.q, 'offset': set_page(page), 'count': '10', 'safesearch': '0', 'device': 'desktop', 'locale': 'en_us'}
		while True:
			self.framework.verbose(f"[QWANT] Searching in {page+1} page...")
			try:
				req = self.framework.request(url=self.qwant, params=payload)
			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/engines/qwant', 'name_crawl')
				self.framework.error('Qwant is missed!', 'util/engines/qwant', 'name_crawl')
				break
			else:
				if req.status_code == 429 and "I can't let you do that..." in req.text and '<div class="error-code">' in req.text:
					self.framework.error('429 Too Many Requests')
					return
				self._pages += req.text
				try:
					self._json.append(req.json())
				except Exception as e:
					self.framework.error('429 Too Many Requests')
					return
				else:
					if req.json() == {'status': 'error', 'data': {'error_code': 22}}:
						self.framework.error('429 Too Many Requests')
						return
					else:
						if page == self.limit:
							break
						page += 1
						payload['offset'] = set_page(page)

	@property
	def pages(self):
		return self._pages

	@property
	def json(self):
		return self._json

	@property
	def links(self):
		for page in self._json:
			results = page.get('data', {}).get('result', {}).get('items', {})
			self._links.extend([x.get('url') for x in results])
		return self._links

	@property
	def results(self):
		results = []
		for page in self._json:
			items = page.get('data', {}).get('result', {})
			if items:
				items = items.get('items', {})
				if items:
					items = items.get('mainline', {})
			for item in items:
				inside_items = item.get('items')
				for i in inside_items:
					a = i['url']
					result = {
						't': i['title'],
						'a': a,
						'c': self.framework.meta_search_util().make_cite(a),
						'd': '' if 'desc' not in i else i.get('desc'),
					}
					results.append(result)
		return results

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q, self.links)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


class main:

	def __init__(self):
		""" show histogram plot of web term frequency """
		self.framework = main.framework

	def line_chart(vectors, title, xlabel, ylabel, **args):
		import matplotlib.pyplot as plt

		for vector in vectors:
			x = [i for i in range(len(vector))]
			plt.plot(x, vector, **args)
		plt.xlabel(xlabel)
		plt.ylabel(ylabel)
		plt.title(title)
		plt.legend()
		plt.show()

	def scatter_chart(vectors, title, xlabel, ylabel, **args):
		import matplotlib.pyplot as plt

		for vector in vectors:
			plt.scatter(vector[0], vector[1], **args)
		plt.xlabel(xlabel)
		plt.ylabel(ylabel)
		plt.title(title)
		plt.legend()
		plt.show()



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, q):
		"""
		openstreetmap search engine for places
		q       :   query
		limit   :   max result count
		"""

		self.framework = main.framework
		self.q = self.framework.urlib(q).quote
		self._pages = ''
		self.xpath_name = {
			'results': '//li[@class="list-group-item search_results_entry"]',
			'results_a': './a',
		}
		self.xpath = {
			self.xpath_name['results']: [
				self.xpath_name['results_a']
			]
		}

	def run_crawl(self):
		urls = [f"https://www.openstreetmap.org/geocoder/search_geonames?query={self.q}",
				f"https://www.openstreetmap.org/geocoder/search_osm_nominatim?query={self.q}"]
		self.framework.verbose('Searching openstreetmap...')

		for url in urls:
			try:
				req = self.framework.request(url=url)
			except Exception as e:
				self.framework.error(f"ConnectionError {e}", 'util/openstreetmap', 'run_crawl')
				self.framework.error('Openstreetmap is missed!', 'util/openstreetmap', 'run_crawl')
				return
			else:
				self._pages += req.text

	@property
	def pages(self):
		return self._pages

	@property
	def results(self):
		results = []
		parser = self.framework.page_parse(self._pages)
		xpath_results = parser.html_fromstring(self.xpath)
		if not xpath_results:
			return results
		root = xpath_results[self.xpath_name['results']]
		for i in range(len(root[self.xpath_name['results_a']])):
			if root[self.xpath_name['results_a']][i].get('data-prefix') is not None:
				category = root[self.xpath_name['results_a']][i].get('data-prefix')
			else:
				category = ''
			link = f"https://www.openstreetmap.org{root[self.xpath_name['results_a']][i].get('href')}"
			result = {
				'category': category,
				'location': root[self.xpath_name['results_a']][i].get('data-name'),
				'latitude': root[self.xpath_name['results_a']][i].get('data-lat'),
				'longitude': root[self.xpath_name['results_a']][i].get('data-lon'),
				'link': link
			}
			results.append(result)

		return results



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import html
import urllib.parse

class main:

		def __init__(self, q, limit=15):
			""" activesearchresults.com search engine

					q     : Query for search
					limit : Maximum result count
			"""
			self.framework = main.framework
			self.q = q
			self._max = limit
			self._cookie = ''
			self._rawhtml = ''
			self._clubbedrows = []
			self._rows = []
			self._links = []
			self._links_with_data = []
			self._pageno = 0
			self._soup = ''
			self.active_search = 'www.activesearchresults.com'
			

		def run_crawl(self):
			from bs4 import BeautifulSoup as bs

			payload = { 
				'wordsall': self.q,
				'asrtech' : 'yes',
				'perpage' : '100',
				'adultfilter': '5',
				'buttonall': ''
				}
			header = {
				'Cookie' : f'{self._cookie}'
				}

			self.framework.verbose('Searching the activesearchresults domain...')

			while True:
				self._pageno += 1
				url = f'https://{self.active_search}/searchsubmit.php?pageno={self._pageno}'

				try:
					req = self.framework.request(url=url,
						method='POST',
						data=payload,
						headers=header
						)
				except:
					self.framework.error('ConnectionError', 'util/activesearch', 'run_crawl')
					self.framework.error('activesearchresults is missed!', 'util/activesearch', 'run_crawl')
					return

				self._rawhtml += req.text

				if self._pageno==1:
					max_pages = list(map(int, re.findall(
						r"<a href='/searchsubmit.php\?pageno=(\d+[^']+)'>LAST",
						req.text)))

					if len(max_pages)==0:
						return

					else:
						max_pages = max_pages[0]

					self._cookie = req.headers.get('Set-Cookie')


				if self._pageno > max_pages or self._pageno*100>self._max:
					break

			self._soup = bs(self._rawhtml,'html.parser')
			self._clubbedrows = self._soup.find_all('small')[5]
			self._rows = self._clubbedrows.find_all('a')

			for count, row in enumerate(self._rows):
				if count>=self._max:
					break
				self._links.append(row['href'])
				self._links_with_data.append({
					'title': row['href'],
					'link' : row.text
					})

		@property
		def raw(self):
			return self._rawhtml

		@property
		def rows(self):
			return self._rows

		@property
		def links(self):
			return self._links

		@property
		def dns(self):
			return self.framework.page_parse(self._rawhtml).get_dns(self.q)

		@property
		def emails(self):
			return self.framework.page_parse(self._rawhtml).get_emails(self.q)

		@property
		def docs(self):
			return self.framework.page_parse(self._rawhtml).get_docs(self.q, self.links)

		@property
		def links_with_data(self):
			return self._links_with_data 



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import datetime

class main:

	def __init__(self, q, count=20, sortby='relevance', verbose=False):
		""" Reddit search engine

				q         : query for search
				count     : maximum result count
				verbose   : print entire json 
		"""
		self.framework = main.framework
		self.q = q
		self.count = count
		self.sortby = sortby
		self._json = {}
		self._posts = []
		self._verbose = verbose

	def run_crawl(self):
		self.framework.verbose('[REDDIT] Searching the reddit domain...')
		url = f'https://gateway.reddit.com/desktopapi/v1/search'
		payload = {
			'q': self.q,
			'sort': self.sortby,
			't': 'all'
		}

		for _ in range(self.count//25+1):
			try:
				res = self.framework.request(
						url, 
						params=payload,
				)

			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/osint/reddit', 'run_crawl')
				self.framework.error('Reddit is missed!', 'util/osint/reddit', 'run_crawl')

			else:
				self._json.update(res.json())
				self._posts.extend(list(self._json['posts'].values()))

	@property
	def json(self):
		return self._json

	@property
	def posts(self):
		return self._posts

	@property
	def results(self):
		results = []
		for count, post in enumerate(self._posts, 1):
			if count > self.count:
				break

			date = datetime.datetime.fromtimestamp(int(post['created'])//1000).strftime("%m/%d/%Y, %H:%M:%S")

			media = post['media']
			if media is not None:
				type = media['type']

				if type == 'embed':
					desc = media['content']

				elif type == 'text':
					desc = media.get('markdownContent')

				elif type == 'video':
					desc = 'Video description currently not supported'

				elif type == 'gallery':
					desc = []
					for item in media['gallery']['items']:
						id = item['mediaId']
						desc.append(media['mediaMetadata'][id]['s']['u'])
					desc = '\n'.join(desc)

				elif type == 'image':
					desc = media['content']

			elif post['source'] is not None:
				desc = post['source']['url']

			else:
				desc = 'Description not available'

			results.append({
				't': post['title'],
				'a': post['permalink'],
				'c': f"author: {post['author']} | date: {date}",
				'd': desc
			})

		return results




#!/usr/bin/env python3
"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:
	"""
	A scikit-learn estimator for TfidfRetriever. Trains a tf-idf matrix from a corpus
	of documents then finds the most N similar documents of a given input document by
	taking the dot product of the vectorized input document and the trained tf-idf matrix.

	Parameters
	----------
	lowercase : boolean
		Convert all characters to lowercase before tokenizing. (default is True)
	preprocessor : callable or None
		Override the preprocessing (string transformation) stage while preserving
		the tokenizing and n-grams generation steps. (default is None)
	tokenizer : callable or None
		Override the string tokenization step while preserving the preprocessing
		and n-grams generation steps (default is None)
	stop_words : string {‘english’}, list, or None
		If a string, it is passed to _check_stop_list and the appropriate stop
		list is returned. ‘english’ is currently the only supported string value.
		If a list, that list is assumed to contain stop words, all of which will
		be removed from the resulting tokens.
		If None, no stop words will be used. max_df can be set to a value in the
		range [0.7, 1.0) to automatically detect and filter stop words based on
		intra corpus document frequency of terms.
		(default is None)
	token_pattern : string
		Regular expression denoting what constitutes a “token”. The default regexp
		selects tokens of 2 or more alphanumeric characters (punctuation is completely
		ignored and always treated as a token separator).
	ngram_range : tuple (min_n, max_n)
		The lower and upper boundary of the range of n-values for different n-grams
		to be extracted. All values of n such that min_n <= n <= max_n will be used.
		(default is (1, 1))
	max_df : float in range [0.0, 1.0] or int
		When building the vocabulary ignore terms that have a document frequency strictly
		higher than the given threshold (corpus-specific stop words). If float, the parameter
		represents a proportion of documents, integer absolute counts. This parameter is
		ignored if vocabulary is not None. (default is 1.0)
	min_df : float in range [0.0, 1.0] or int
		When building the vocabulary ignore terms that have a document frequency
		strictly lower than the given threshold. This value is also called cut-off
		in the literature. If float, the parameter represents a proportion of
		documents, integer absolute counts. This parameter is ignored if vocabulary
		is not None. (default is 1)
	vocabulary : Mapping or iterable, optional
		Either a Mapping (e.g., a dict) where keys are terms and values are indices
		in the feature matrix, or an iterable over terms. If not given, a vocabulary
		is determined from the input documents. (default is None)
	paragraphs : iterable
		an iterable which yields either str, unicode or file objects
	top_n : int (default 20)
		maximum number of top articles (or paragraphs) to retrieve
	verbose : bool, optional
		If true, all of the warnings related to data processing will be printed.

	Attributes
	----------
	vectorizer : TfidfVectorizer
		See https://scikit-learn.org/stable/modules/generated/sklearn.feature_extraction.text.TfidfVectorizer.html
	tfidf_matrix : sparse matrix, [n_samples, n_features]
		Tf-idf-weighted document-term matrix.

	Examples
	--------
	>>> retriever = TfidfRetriever(ngram_range=(1, 2), max_df=0.85, stop_words='english')
	>>> retriever.fit(X=df)
	>>> best_idx_scores = retriever.predict(X='A question or text for which retriever will find closest match')
	"""

	def __init__(
		self,
		lowercase=True,
		preprocessor=None,
		tokenizer=None,
		stop_words="english",
		token_pattern=r"(?u)\b\w\w+\b",
		ngram_range=(1, 2),
		max_df=1,
		min_df=0,
		vocabulary=None,
		top_n=20,
		verbose=False,
	):
		from sklearn.feature_extraction.text import TfidfVectorizer

		self.framework = main.framework
		self.lowercase = lowercase
		self.preprocessor = preprocessor
		self.tokenizer = tokenizer
		self.stop_words = stop_words
		self.token_pattern = token_pattern
		self.ngram_range = ngram_range
		self.max_df = max_df
		self.min_df = min_df
		self.vocabulary = vocabulary
		self.top_n = top_n
		self.verbose = verbose

		vectorizer = TfidfVectorizer(
			lowercase=self.lowercase,
			preprocessor=self.preprocessor,
			tokenizer=self.tokenizer,
			stop_words=self.stop_words,
			token_pattern=self.token_pattern,
			ngram_range=self.ngram_range,
			max_df=self.max_df,
			min_df=self.min_df,
			vocabulary=self.vocabulary,
		)
		self.vectorizer = vectorizer

	def fit(self, df, y=None):
		self.metadata = df
		self.tfidf_matrix = self.vectorizer.fit_transform(list(map(' '.join, df["pages"])))
		return self

	def predict(self, query: str) -> 'OrderedDict':
		"""
		Compute the top_n closest documents given a query

		Parameters
		----------
		query: str

		Returns
		-------
		best_idx_scores: OrderedDict
			Dictionnaire with top_n best scores and idices of the documents as keys

		"""
		import time
		import prettytable
		from collections import OrderedDict

		t0 = time.time()
		scores = self._compute_scores(query)
		idx_scores = [(idx, score) for idx, score in enumerate(scores)]
		best_idx_scores = OrderedDict(
			sorted(idx_scores, key=(lambda tup: tup[1]), reverse=True)[: self.top_n]
		)
		closest_docs_indices = list(best_idx_scores.keys())

		if self.verbose:
			rank = 1
			table = prettytable.PrettyTable(["rank", "index", "title"])
			for i in range(len(closest_docs_indices)):
				index = closest_docs_indices[i]
				# if self.paragraphs:
				#	  article_index = self.paragraphs[int(index)]["index"]
				#	  title = self.metadata.iloc[int(article_index)]["title"]
				# else:
				title = self.metadata.iloc[int(index)]["title"]
				table.add_row([rank, index, title])
				rank += 1
			print(table)
			print("Time: {} seconds".format(round(time.time() - t0, 5)))

		return best_idx_scores

	def _compute_scores(self, query):
		question_vector = self.vectorizer.transform([query])
		scores = self.tfidf_matrix.dot(question_vector.T).toarray()
		return scores

	def filter_pages(
		self,
		articles,
		drop_empty=True,
		read_threshold=1000,
		public_data=True,
		min_length=0,
		max_length=5000,
	):
		"""
		Cleans the pages and filters them regarding their length
		Parameters
		----------
		articles : DataFrame of all the articles 
		Returns
		-------
		Cleaned and filtered dataframe
		Examples
		--------
		>>> df = pd.read_csv('data.csv')
		>>> df_cleaned = filter_pages(df)
		"""

		from ast import literal_eval

		# Replace and split
		def replace_and_split(pages):
			for page in pages:
				page.replace("'s", " " "s").replace("\\n", "").split("'")
			return pages

		# Select pages with the required size
		def filter_on_size(pages, min_length=min_length, max_length=max_length):
			page_filtered = [
				page.strip()[:max_length]
				for page in pages
			]
			return page_filtered

		# Cleaning and filtering
		articles["pages"] = articles["pages"].apply(lambda x: literal_eval(str(x)))
		articles["pages"] = articles["pages"].apply(replace_and_split)
		articles["pages"] = articles["pages"].apply(filter_on_size)
		articles["pages"] = articles["pages"].apply(
			lambda x: x if len(x) > 0 else np.nan
		)

		# Read threshold for private dataset
		if not public_data:
			articles = articles.loc[articles["number_of_read"] >= read_threshold]

		# Drop empty articles
		if drop_empty:
			articles = articles.dropna(subset=["pages"]).reset_index(drop=True)

		return articles



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import json
import concurrent.futures

from maryam.core.basedir import BASEDIR

meta = {
	'name': 'Username Search',
	'author': 'Aman Singh',
	'version': '0.5',
	'description': 'Search your query across 100+ social networks and show the results.',
	'sources': ('https://github.com/sherlock-project/sherlock',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('thread', 64, False, 'The number of sites that are being checked per round(default=8)', '-t', 'store', int),
	),
	'examples': ('username_search -q <QUERY> --output',)
}

OUTPUT = {'links': {}}

def thread(self, data, query,thread_count):
	threadpool = concurrent.futures.ThreadPoolExecutor(max_workers=thread_count)
	futures = (threadpool.submit(check, self, data[site]['url'].format(self.urlib(query).quote_plus.replace('.', '%2E')), site, data) for site in data)
	for results in concurrent.futures.as_completed(futures):
		print(f"Found {len(OUTPUT['links'])} accounts" , end='\r')
	print('\n')


def check(self, url, site, data):
	global OUTPUT
	try:
		headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
		req = self.request(url, headers=headers, timeout=20)
	except Exception as e:
		return
	else:
		if str(req.status_code) == data[site]['status']:
			for error in data[site]['error']:
				if error in req.text :
					return
		elif str(req.status_code)[0] == '2' or str(req.status_code)[0] == '3':
			OUTPUT['links'][site] = {'url': url, 'rank': data[site]['rank']}

def module_api(self):
	query = self.options['query']
	filepath = os.path.join(BASEDIR,
				'data',
				'username_checker.json')
	with open(filepath) as handle:
		data = json.load(handle)
	thread(self, data, query,self.options['thread'])
	output = OUTPUT
	self.save_gather(
		output,
		'osint/username_search',
		query,
		output=self.options.get('output')
	)
	output['links'] = sorted(
		list(output['links'].items()), key=lambda x: int(x[1]['rank'])
	)
	return output

def module_run(self):
	output = module_api(self)

	sites = [(name, meta['url']) for name, meta in output['links']]

	self.alert("Accounts Found (sorted by site's rank)")
	self.table(sites , header=['Site', 'Account'], linear=True, sep='_')

def refresh_username_checker_siteranks():
	"""
	Function which refreshes the `rank` properity in `username_checker.json`.
	This method should only be called manually.
	To fetch data you need an API key. To obtain it, register at:
	https://www.domcop.com/openpagerank/
	"""
	import requests
	from urllib.parse import urlparse

	API_KEY = input('Please input your OpenSiteRank API key: ')

	def getRank(url):
		header = {'API-OPR': API_KEY}
		req_url = f"https://openpagerank.com/api/v1.0/getPageRank?domains[]={url}"
		r = requests.get(req_url, headers=header)
		result = json.loads(r.text)	
		return result

	data = dict()

	filepath = os.path.join(BASEDIR,
				'data',
				'username_checker.json')

	with open(filepath, 'r') as f:
		data = json.load(f) 

	for site in data:
		domain = urlparse(data[site]['url']).netloc.replace('{}.', '')
		rank = getRank(domain)
		if rank['status_code'] != 200:
			print('\nERROR:\n', rank)
		else:
			data[site]['rank'] = rank['response'][0]['rank']

	with open(filepath, 'w') as f:
		f.write(json.dumps(data))




"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import plotly

class main:

	def __init__(self):
		""" a class for plotly framework. currently it just supports common methods """
		self.framework = main.framework

	def tf_pie(self, docs: 'documents', form: 'documet form. e.g html',
				last: 'number of words', without_punc=True, remove_stopwords=False):
		tf = self.framework.tf_histogram(docs, form, without_punc)
		if remove_stopwords:
			tf.remove_stopwords()
		bow = tf._counter(last)
		values = []
		labels = []
		for word, count in bow:
			values.append(count)
			labels.append(word)
		fig = plotly.graph_objects.Figure(plotly.graph_objects.Pie(values=values,labels=labels,
			texttemplate = "%{label}: %{value:s} <br>(%{percent})",
			textposition = "inside"))
		fig.show()



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import cmd
import json
import os
import re
import subprocess
import sys
import traceback
import requests
from .util.helpers import rand_uagent
from textwrap import wrap

class FrameworkException(Exception):
	def __init__(self, message):
		Exception.__init__(self, message)

class Colors:
	N = '\033[m'  # native
	R = '\033[91m'  # red
	G = '\033[92m'  # green
	O = '\033[93m'  # orange
	B = '\033[94m'  # blue
	P = '\033[95m'  # purple
	C = '\033[0;1;36m'  # cyan
	Y = '\u001b[38;5;226m'

class core(cmd.Cmd):
	prompt = '>>>'
	_global_options = {}
	_global_options_ = {}
	_loaded_modules = {}
	_cat_module_names = {}
	_module_names = []
	_error_stack = []
	workspace = ''
	Colors = Colors

	def __init__(self):
		cmd.Cmd.__init__(self)
		self.ruler = '-'
		self.spacer = '  '
		self.nohelp = f"{Colors.R}[!] No help on %s{Colors.N}"
		self.do_help.__func__.__doc__ = 'Displays this menu'
		self.doc_header = 'Commands (type [help|?] <topic>):'
		self._exit = 0

	# ////////////////////////////////
	#             OVERRIDE          //
	# ////////////////////////////////

	def default(self, line):
		self.do_shell(line)

	def emptyline(self):
		return 0

	def precmd(self, line):
		line = self.to_str(line)
		return line

	def onecmd(self, line):
		line = self.to_str(line)
		cmd, arg, line = self.parseline(line)
		if not line:
			return self.emptyline()
		if line == 'EOF':
			sys.stdin = sys.__stdin__
			core._script = 0
			core._load = 0
			return 0
		if cmd is None:
			return self.default(line)
		self.lastcmd = line
		if cmd == '':
			return self.default(line)
		else:
			if cmd in self._module_names:
				return self.run_tool('opt_proc', cmd, arg)
			else:
				try:
					func = getattr(self, 'do_' + cmd)
				except AttributeError:
					return self.default(line)
				try:
					return func(arg)
				except Exception as e:
					self.print_exception()

	def print_topics(self, header, cmds, cmdlen, maxcol):
		if cmds:
			self.stdout.write(f"{header}{os.linesep}")
			if self.ruler:
				self.stdout.write(f"{self.ruler * len(header)}{os.linesep}")
			for cmd in cmds:
				self.stdout.write(f"{cmd.ljust(15)} {getattr(self, 'do_' + cmd).__doc__}{os.linesep}")
			self.stdout.write(os.linesep)

	# ////////////////////////////////
	#           SUPPORT             //
	# ////////////////////////////////

	def to_str(self, obj):
		if not isinstance(obj, str):
			if isinstance(obj, bytes):
				obj = str(obj, 'utf-8')
			else:
				obj = str(obj)
		return obj

	def _is_readable(self, filename, flag='r'):
		try:
			return open(filename, flag)
		except IOError as e:
			self.error('IOError: ' + str(e))
			return False

	def _check_version(self):
		if not self._global_options.get('update_check'):
			return
		self.debug('Checking version...')
		pattern = r"__VERSION__\s+=\s+'(\d+\.\d+\.\d+[^']*)'"
		remote = 0
		local = 0
		try:
			remote = re.search(pattern, self.request(\
				'https://raw.githubusercontent.com/saeeddhqan/maryam/master/maryam/__main__.py').text).group(1)
			local = self.__version__[1:]
		except Exception as e:
			self.error(f"Version check failed ({type(e).__name__}).")
		if remote != local:
			self.alert('Your version of Maryam does not match the latest release')
			self.output(f"Remote version:  {remote}")
			self.output(f"Local version:   {local}")

	# ////////////////////////////////
	#           OUTPUT              //
	# ////////////////////////////////

	def print_exception(self, line='', where='nil', which_func='nil'):
		stack_list = [x.strip() for x in traceback.format_exc().strip().splitlines()]
		message = stack_list[-1]
		if self._global_options['verbosity'] == 0:
			return
		elif self._global_options['verbosity'] == 1:
			line = ' '.join([x for x in [message, line] if x])
			self.error(stack_list[-3], where, which_func)
			self.error(line, where, which_func)
		elif self._global_options['verbosity'] >= 2:
			print(f"{Colors.R}{'-'*60}")
			traceback.print_exc()
			print(f"{'-'*60}{Colors.N}")

	def textwrapping(self, prefix, string, size=80):
		size -= len(prefix)
		if isinstance(string, bytes):
			string = ''.join(r'\x{:02x}'.format(byte) for byte in string)
			if size % 2:
				size -= 1
		return '\n'.join([prefix + line for line in wrap(string, size)])

	def error(self, line, where='nil', which_func='nil'):
		'''Formats and presents errors.'''
		if not re.search(r'[.,;!?]$', line):
			line += '.'
		line = line[:1].upper() + line[1:]
		error = f"[{where}:{which_func}] {line}"
		print(f"{Colors.O}[!] {error}{Colors.N}")
		if error not in self._error_stack:
			self._error_stack.append(error)

	def output(self, line, color='N', end='', prep='', linesep=True, prefix='[*]'):
		'''Formats and presents normal output.'''
		line = self.to_str(line)
		line = f'{prep}{Colors.B}{prefix}{getattr(Colors, color.upper())} {line}\033[m'
		if not line.endswith(os.linesep) and linesep:
			line += os.linesep
		print(line, end=end, flush=True)

	def alert(self, line):
		'''Formats and presents important output.'''
		print(f"{Colors.G}[*]{Colors.N} {line}")

	def verbose(self, line, color='n', end=''):
		'''Formats and presents output if in verbose mode.'''
		if self._global_options['verbosity'] >= 1:
			self.output(line, color, end)

	def debug(self, line, end='', linesep=True):
		'''Formats and presents output if in debug mode (very verbose).'''
		if self._global_options['verbosity'] >= 2:
			line = f'{Colors.C}[~] {Colors.N}{self.to_str(line)}'
			if not line.endswith(os.linesep) and linesep:
				line += os.linesep
			print(line, end=end)

	def heading(self, line, level=1):
		'''Formats and presents styled header text'''
		line = line
		print('')
		if level == 0:
			print(self.ruler*len(line))
			print(line.upper())
			print(self.ruler*len(line))
		if level == 1:
			print(f"{self.spacer}{line.title()}")
			print(f"{self.spacer}{self.ruler*len(line)}")

	def table(self, data, header, title='', linear=False, sep='-'):
		tdata = list(data)
		if header:
			tdata.insert(0, header)
		if len(set([len(x) for x in tdata])) > 1:
			raise FrameworkException('Row lengths not consistent.')
		lens = []
		cols = len(tdata[0])
		for i in range(0, cols):
			lens.append(len(max([self.to_str(x[i]) if x[i] != None else '' for x in tdata], key=len)))
		title_len = len(title)
		tdata_len = sum(lens) + (3*(cols-1))
		diff = title_len - tdata_len
		if diff > 0:
			diff_per = diff // cols
			lens = [x+diff_per for x in lens]
			diff_mod = diff % cols
			for x in range(0, diff_mod):
				lens[x] += 1
		if len(tdata) > 0:
			separator_str = f"{self.spacer}+{sep}{f'%s{sep*3}'*(cols-1)}%s{sep}+"
			separator_sub = tuple([sep*x for x in lens])
			separator = separator_str % separator_sub
			data_str = f"{self.spacer}| {'%s | '*(cols-1)}%s |"
			print('')
			print(separator)
			if title:
				print(f"{self.spacer}| {title.center(tdata_len)} |")
				print(separator)
			if header:
				rdata = tdata.pop(0)
				data_sub = tuple([rdata[i].center(lens[i]) for i in range(0,cols)])
				print(data_str % data_sub)
				print(separator)
			for rdata in tdata:
				data_sub = tuple([self.to_str(rdata[i]).ljust(lens[i]) if rdata[i] != None else ''.ljust(lens[i]) for i in range(0,cols)])
				print(data_str % data_sub)
				if linear:
					print(separator)
			if not linear:
				print(separator)
			print('')

	# ////////////////////////////////
	#             EXPORT            //
	# ////////////////////////////////

	def save_gather(self, value, module, target, method=None, output=True):
		if method is None:
			method = []
		if not output:
			return
		self.debug('Saving data to the gather file...')
		gather_file = os.path.join(self.workspace, 'gather.dat')
		# create the file if one doesn't exist
		if not os.path.exists(gather_file):
			open(gather_file, 'a').close()
			data = {}
		else:
			file =self._is_readable(gather_file)
			try:
				data = json.loads(file.read())
			except ValueError:
				data = {}

		if module in data:
			if target in data[module]:
				if method == []:
					data[module][target] = value
				else:
					for i in method:
						if i in data[module][target]:
							data[module][target][i] = value[i]
						else:
							data[module][target].update({i: value[i]})
			else:
				data[module].update({target: value})
		else:
			data.update({module: {target: value}})
		# update gather file
		file = self._is_readable(gather_file, 'w')
		json.dump(data, file, indent=4)
		self.debug(f"{file.name} => Done")

	def json2xml(self, json_obj, line_padding=''):
		result_list = list()

		if isinstance(json_obj, list):
			for sub_elem in json_obj:
				result_list.append(self.json2xml(sub_elem, line_padding))
			return os.linesep.join(result_list)

		if isinstance(json_obj, dict):
			for tag_name in json_obj:
				sub_obj = json_obj[tag_name]
				tag_name = re.sub(r"[\W]+", '_', tag_name)
				result_list.append(f"{line_padding}\t<{tag_name}>")
				result_list.append(self.json2xml(sub_obj, '\t' + line_padding).replace('&','&amp;'))
				result_list.append(f"{line_padding}\t</{tag_name}>")

			return f"{os.linesep}{os.linesep.join(result_list)}{os.linesep}"

		return f'\t{line_padding}{json_obj}'

	def json2csv(self, json_obj, separator=';', parent='.'):
		titles = list(json_obj.keys())
		resp = {}
		if isinstance(json_obj[titles[0]], list):
			for title in titles:
				resp[f"{parent}/{title}"] = json_obj[title]
			return resp
		else:
			for title in titles:
				obj = json_obj[title]
				if isinstance(obj, list):
					resp[f"{parent}/{title}"] = obj
				elif isinstance(obj, dict):
					tmp = self.json2csv(obj, separator, parent+'/'+title)
					resp.update(tmp)
			return resp

	def csv2text(self, json_obj, separator=';'):
		get_titles = list(json_obj.keys())
		first = f"{separator.join(get_titles)}\n"
		for tup in range(len(json_obj[max(json_obj, key=len)])):
			line = []
			for atom in get_titles:
				atom = json_obj[atom]
				atom = atom[tup] if tup < len(atom) else False
				if atom:
					line.append(atom)
			first += f"{separator.join(line)}\n"
		first = first[:-1]
		return first

	def exporter(self, datasets, filename, method):
		"""Export the results."""

		# Initialize filename
		if '/' not in filename:
			filename = os.path.join(self.workspace, filename)
			if f'.{method}' not in filename.lower():
				filename = f'{filename}.{method}'
		self.debug(f'Making a report with {method} format and save to the {filename}')
		file = self._is_readable(filename, 'w+')
		if file:
			if method == 'json':
				# Convert json_dict to a JSON styled string
				json_string = json.dumps(datasets, indent=4)
				file.write(json_string)
			elif method == 'csv':
				csv = self.json2csv(datasets)
				text = self.csv2text(csv)
				file.write(text)
			elif method == 'xml':
				text = self.json2xml(datasets)
				text = f'<?xml version="1.0" encoding="UTF-8"?>\n{text}'
				file.write(text)
			else:
				# Default method is txt
				json_string = json.dumps(datasets, indent=4)
				json2txt = re.sub(r"[\"']+", '', str(json_string))
				file.write(json2txt)

			file.close()
			return filename
		return False

	# ////////////////////////////////
	#           ERRORS   			//
	# ////////////////////////////////

	def _reset_error_stack(self):
		self._error_stack.clear()

	# ////////////////////////////////
	#           OPTIONS 			//
	# ////////////////////////////////

	def _load_config(self):
		config_path = os.path.join(self.workspace, 'config.dat')

		if os.path.exists(config_path):
			config_file = self._is_readable(config_path)
			try:
				config_data = self.config_data = json.loads(config_file.read())
			except ValueError:
				pass
			else:
				for key in self._global_options:
					try:
						self._global_options[key] = config_data[key]
					except KeyError:
						continue

	def _save_config(self, name):
		config_path = os.path.join(self.workspace, 'config.dat')
		if not os.path.exists(config_path):
			self._is_readable(config_path, 'a').close()
			config_data = {}
		else:
			with open(config_path) as config_file:
				try:
					config_data = json.loads(config_file.read())
				except ValueError:
					config_data = {}


		config_data[name] = self._global_options[name]

		if config_data[name] is None:
			del config_data[name]

		with open(config_path, 'w') as config_file:
			json.dump(config_data, config_file, indent=4)

	# ////////////////////////////////
	#           request             //
	# ////////////////////////////////

	def _print_prepared_request(self, prepared):
		self.debug(f"{'='*25} REQUEST {'='*25}{os.linesep}")
		print(f"url:    {prepared.url}")
		print(f"method: {prepared.method} {prepared.path_url}")
		for k, v in prepared.headers.items():
			print(f"header: {k}: {v}")
		if prepared.body:
			print(f"body: {prepared.body}")

	def _print_response(self, resp):
		self.debug(f"{'='*25} RESPONSE {'='*25}{os.linesep}")
		print(f"status: {resp.status_code} {resp.reason}")
		for k, v in resp.headers.items():
			print(f"header: {k}: {v}")

	def request(self, url, method='GET', **kwargs):
		if '://' not in url:
			url = f"{self._global_options['protocol']}://{url}"
		kwargs['timeout'] = kwargs.get('timeout') or self._global_options['timeout']
		headers = kwargs.get('headers') or {}
		if self._global_options['rand_agent']:
			headers['user-agent'] = rand_uagent.main().get
		else:
			headers['user-agent'] = headers.get('user-agent', False) or self._global_options['agent']
		headers = {k.title(): v for k, v in headers.items()}
		kwargs['headers'] = headers
		# process proxy
		proxy = self._global_options['proxy']
		if proxy:
			proxies = {
				'http': f"http://{proxy}",
				'https': f"http://{proxy}",
			}
			kwargs['proxies'] = proxies
		kwargs['verify'] = False
		requests.packages.urllib3.disable_warnings(requests.packages.urllib3.exceptions.InsecureRequestWarning)
		resp = getattr(requests, method.lower())(url, **kwargs)
		if self._global_options['verbosity'] < 2:
			return resp
		self._print_prepared_request(resp.request)
		self._print_response(resp)
		return resp

	# ////////////////////////////////
	#           SHOW                //
	# ////////////////////////////////

	def show_modules(self, param):
		for section in core._cat_module_names:
			self.heading(section)
			for module in core._cat_module_names[section]:
				print(f'{self.spacer * 2}{module}')
		print('')

	def show_options(self, options=None):
		'''Lists options'''
		if options is None:
			options = self._global_options_
		if options:
			pattern = f'{self.spacer}%s  %s  %s  %s'
			key_len = len(max(options, key=len))
			if key_len < 4:
				key_len = 4
			val_len = len(max([self.to_str(self._global_options[x]) for x in options], key=len))
			if val_len < 13:
				val_len = 13
			print('')
			print(pattern % ('Name'.ljust(key_len), 'Current Value'.ljust(val_len), 'Required', 'Description'))
			print(pattern % (self.ruler*key_len, (self.ruler*13).ljust(val_len), self.ruler*8, self.ruler*11))
			for key in sorted(options):
				option = options[key]
				value = self._global_options[key] if self._global_options[key] != None else ''
				reqd = 'no' if not option[2] else 'yes'
				desc = option[3]
				print(pattern % (key.ljust(key_len), \
					self.to_str(value).ljust(val_len), \
					self.to_str(reqd).ljust(8), desc.title()))
			print('')
		else:
			print(f'{os.linesep}{self.spacer}No options available for this module.{os.linesep}')

	def _get_show_names(self):
		prefix = 'show_'
		return [x[len(prefix):]
				for x in self.get_names() if x.startswith(prefix)]

	# /////////////////////
	#         DEV       //
	# ////////////////////

	def _dev_install_requirements(self, reqs):
		'''Install requirements with pip.'''
		well = []
		for req in reqs:
			well.append(not bool(self.do_shell(f"{sys.executable} -m pip install {req}")))
		return all(well)

	# ////////////////////////////////
	#           COMMANDS            //
	# ////////////////////////////////

	def do_exit(self, params):
		'''Exits the framework'''
		self._exit = 1
		return True

	def do_set(self, params):
		'''Sets module options'''
		options = params.split()
		if len(options) < 2:
			self.help_set()
			return
		name = options[0].lower()
		if name in self._global_options:
			value = ' '.join(options[1:])
			if self._global_options_[name][2] and (not value or value == 'None'):
				print(f"{name} is a required option.")
				return
			if isinstance(self._global_options[name], bool):
				if value.lower() in ('true', 'yes', 'on'):
					value = True
				elif value.lower() in ('false', 'no', 'off'):
					value = False
				else:
					print(f"{name} is a bool option. got {value}")
					return
				self._global_options[name] = value
			elif isinstance(self._global_options[name], int):
				if value.isdigit():
					self._global_options[name] = int(value)
				else:
					print(f"{name} is an int option. got {value}")
					return
			else:
				if value == 'None':
					value = None
				self._global_options[name] = value
			print(f"{name.upper()} => {value}")
			self._save_config(name)
		else:
			self.error('Invalid option.')

	def do_unset(self, params):
		'''Unsets module options'''
		self.do_set(f"{params} {None}")

	def do_show(self, params):
		'''Shows various framework items'''
		if not params:
			self.help_show()
			return
		_params = params
		params = params.lower().split()
		arg = params[0]
		params = ' '.join(params[1:])
		if arg in self._get_show_names():
			func = getattr(self, f"show_{arg}")
			if arg == 'modules':
				func(params)
			else:
				func()
		else:
			self.help_show()

	def do_search(self, params):
		'''Searches available modules'''
		if not params:
			self.help_search()
			return
		text = params.split()[0]
		self.output(f"Searching for '{text}'...")
		modules = [x for x in core._loaded_modules if text in x]
		if not modules:
			self.error(f"No modules found containing '{text}'.")
		else:
			for section in core._cat_module_names:
				self.heading(section)
				for mod in core._cat_module_names[section]:
					if text in mod:
						self.output(f"\tFound '{text}' under {section}: {mod}", prefix='')

	def do_shell(self, params):
		'''Executes shell commands'''
		if not params:
			self.help_shell()
			return
		proc = subprocess.Popen(params, shell=True, stdout=subprocess.PIPE,
								stderr=subprocess.PIPE, stdin=subprocess.PIPE)
		self.output(f"Command: {params}{os.linesep}")
		stdout = proc.stdout.read()
		stderr = proc.stderr.read()
		if stdout:
			print(f"{Colors.O}{self.to_str(stdout)}{Colors.N}", end='')
		if stderr:
			print(f"{Colors.R}{self.to_str(stderr)}{Colors.N}", end='')
		return stderr

	def do_report(self, params):
		'''Get report from the Gathers and save it to the other formats'''
		temp_dic={}
		if not params:
			self.help_report()
			return
		arg = params.lower().split(' ')
		gather_file = os.path.join(self.workspace, 'gather.dat')
		if not os.path.exists(gather_file):
			self.alert('No data found.')
			return
		# open gather file
		with open(gather_file) as gf:
			try:
				gather_data = json.loads(gf.read())
			except ValueError:
				self.error('Gather data is incorrect. Gather is missed!') 
				return

		if arg[0] == 'saved':
			if gather_data:
				for mod in gather_data:
					self.alert(mod)
					for q in gather_data[mod]:
						print(f'\t{q}')
			else:
				self.output('No result found.')
			print('')
			return

		if len(arg) < 3 or len(arg) > 4:
			self.error("Module name not found")
			self.help_report()
			return
		_format = arg[0]
		if _format not in ('json', 'txt', 'xml', 'csv'):
			self.error(f"Format '{_format}' doesn't found.")
			return
		mod_name = arg[2].lower()
		# Not supperted csv export
		if mod_name in ('footprint/entry_points',) and _format == 'csv':
			self.error('CSV format does not support entry_points output data.')
			return
			
		if mod_name in gather_data:
			output = gather_data[mod_name]
		else:
			self.error(f"Module '{mod_name}' does not have any data.")
			return

		if len(arg) == 4:
			tar_name = arg[3]
			if tar_name in output:
				temp_dic[tar_name] = output[tar_name]
			else:
				self.error(f"Query name '{tar_name}' is not found.")
				return

		else:
			temp_dic[mod_name]=output

		output_file = os.path.join(self.workspace, arg[1])
		get_export = self.exporter(temp_dic, f"{output_file}.{_format}", _format)
		if get_export:
			self.output(f"Report saved at {get_export}")
	
	def do_update(self, params):
		'''Update modules via module name'''
		if not params:
			self.help_update()
			return
		check = False
		args = params.lower().split()
		if len(args) < 2 or args[0] not in ('module', 'check'):
			self.help_update()
			return

		if args[0] == 'check':
			check = True

		# Means all of modules
		if args[1] == '*':
			modules = self._module_names
		else:
			modules = args[1:]
		for module in modules:
			self.heading(module)
			if module not in self._loaded_modules:
				self.output(f"Module name {module} does not exist.")
				continue
			mod = self._loaded_modules[module]
			file = mod.__file__
			mod_version = mod.meta['version']
			url = f"https://raw.githubusercontent.com/saeeddhqan/Maryam/master/maryam/modules/{'/'.join(file.split('/')[-2:])}"
			text = self.request(url).text
			mod_remote_version = re.search(r"'version'\s*:\s*'([\d\.]+)'\s*,", text)
			if not mod_remote_version:
				self.output(f"Update/check failed ({e}).", prep='\t')
			else:
				mod_remote_version = mod_remote_version.group(1)
				if max(mod_version, mod_remote_version) == mod_remote_version\
					and mod_version != mod_remote_version:
					self.output(f"Remote version: {mod_remote_version}", prep='\t')
					self.output(f"Local version: {mod_version}", prep='\t')
					if not check:
						fopen = self._is_readable(file, 'w')
						if fopen:
							fopen.write(text)
							fopen.close()
							self.output(f"{module} has been updated to {mod_remote_version}.", prep='\t')
						else:
							self.output(f"Update/check failed.", prep='\t')
				else:
					self.output(f"{module} is up to date.", prep='\t')
		self.do_reload()

	# ////////////////////////////////
	#             HELP              //
	# ////////////////////////////////

	def help_report(self):
		print(getattr(self, 'do_report').__doc__)
		print(f'{os.linesep}Usage: report [<format> <filename> [<module_name> or <module_name> <query(hostname,domain name, keywords,etc)>]]')
		print('or       : report [saved] => for show queries')
		print('Examples : report json pdf_docs(without suffix) osint/docs_search company.com')
		print('           report xml pdf_docs(without suffix) osint/docs_search')
		print(f'Formats : xml,json,csv and txt{os.linesep}')

	def help_search(self):
		print(getattr(self, 'do_search').__doc__)
		print(f'{os.linesep}Usage: search <string>{os.linesep}')

	def help_set(self):
		print(getattr(self, 'do_set').__doc__)
		print(f'{os.linesep}Usage: set <option> <value>')
		self.show_options()

	def help_unset(self):
		print(getattr(self, 'do_unset').__doc__)
		print(f'{os.linesep}Usage: unset <option>')
		self.show_options()

	def help_shell(self):
		print(getattr(self, 'do_shell').__doc__)
		print(f"{os.linesep}Usage: [shell|!] <command>")
		print(f"\tor just type a command at the prompt.{os.linesep}")

	def help_show(self):
		options = sorted(self._get_show_names())
		print(getattr(self, 'do_show').__doc__)
		print(f"{os.linesep}Usage: show [{'|'.join(options)}]{os.linesep}")

	def help_update(self):
		print(getattr(self, 'do_update').__doc__)
		print(f"{os.linesep}Usage: update [module|check] <modules or * for all modules>")
		print(f"update check dns_search email_search")
		print(f"update module *")
		print(f"update check *")

	# ////////////////////////////////
	#         AUTOCOMPLETE          //
	# ////////////////////////////////

	def complete_set(self, text, line, begidx, endidx):
		return [x.upper()
				for x in self._global_options if x.upper().startswith(text.upper())]

	complete_unset = complete_set

	def complete_update(self, text, line, begidx, endidx):
		return [x for x in ['check', 'module'] if x.startswith(text.lower())]

	def complete_dev(self, text, line, begidx, endidx):
		return [x for x in ['extension', 'package'] if x.startswith(text.lower())]

	def complete_package(self, text, line, begidx, endidx):
		return [x for x in ['extension', 'repo'] if x.startswith(text.lower())]

	def complete_workspace(self, text, line, begidx, endidx):
		return [x for x in ['add', 'list', 'select'] if x.startswith(text.lower())]

	def complete_show(self, text, line, begidx, endidx):
		options = sorted(self._get_show_names())
		return [x for x in options if x.startswith(text)]



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re


meta = {
	'name': 'TikTok Search',
	'author': 'Prakhar Jain',
	'version': '0.1',
	'description': 'Search your query on TikTok and show the results.',
	'sources': ('google', 'yahoo', 'bing', 'metacrawler', 'millionshort', 'carrot2', 'qwant', 'duckduckgo'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of results per page(min=10, max=100, default=50)', '-c', 'store', int),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
		('engine', 'bing', False, 'Engine names for search(default=bing)', '-e', 'store', str),
	),
	'examples': ('tiktok -q <QUERY> -l 15 --output',)
}

LINKS = []

def search(self, name, q, q_formats, limit, count):
	global LINKS
	engine = getattr(self, name)
	q = q_formats[f"{name}_q"] if f"{name}_q" in q_formats else q_formats['default_q']
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	else:
		attr = engine(q)

	attr.run_crawl()
	LINKS += attr.links

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engines = self.options['engine'].split(',')
	output = {'videos': [], 'usernames': [], 'tags': [], 'music': []}
	q_formats = {
		'default_q': f"site:www.tiktok.com {query}",
		'millionshort_q': f'site:www.tiktok.com "{query}"',
		'qwant_q': f'site:www.tiktok.com {query}'
	}

	self.thread(search, self.options['thread'], engines, query, q_formats, limit, count, meta['sources'])

	links = list(set(LINKS))
	for link in self.reglib().filter(r"https?://(www\.)?tiktok\.com/@", links):
			user_url = re.sub(r"https?://(www\.)?tiktok\.com/", '', link)
			user_url = user_url.rsplit('/')
			user = user_url[0]
			if user not in output['usernames']:
				output['usernames'].append(user)

	for link in self.reglib().filter(r"https?://(www\.)?tiktok\.com/music/", links):
			if link not in output['music']:
				output['music'].append(link)

	for link in self.reglib().filter(r"https?://(www\.)?tiktok\.com/amp/tag/", links):
			if link not in output['tags']:
				output['tags'].append(link)

	for link in self.reglib().filter(r"https?://(www\.)?tiktok\.com/", links):
		if '/video/' in link:
			if link not in output['videos']:
				output['videos'].append(link)


	self.save_gather(output, 'search/tiktok', query, output=self.options.get('output'))
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import lxml

class main:

	def __init__(self, q, count=15):
		""" pubmed.ncbi.nlm.nih.gov search engine

				q         : query for search
				count     : maximum result count
		"""
		self.framework = main.framework
		self.q = q
		self.max = count
		self._rawhtml = ''
		self._results = []
		self.pubmed = 'https://pubmed.ncbi.nlm.nih.gov'
		self.xpath_names = {
			'results': './/article',
			'results_content':'.//div[@class="full-view-snippet"]',
			'results_title':'.//a[@class="docsum-title"]',
			'results_a': './/a[@class="docsum-title"]',
			'results_cite': './/span[@class="docsum-authors full-authors"]'
		}
		self.xpaths = {
			self.xpath_names['results']: [
				self.xpath_names['results_content'],
				self.xpath_names['results_title'],
				self.xpath_names['results_a'],
				self.xpath_names['results_cite']
			]
		}

	def run_crawl(self):
		self.framework.verbose('Searching the pubmed domain...')

		payload = {'term': self.q, 'size': 200}
		try:
			req = self.framework.request(
					url=self.pubmed,
					params=payload)
		except Exception as e:
			self.framework.error(f"ConnectionError {e}.", 'util/pubmed', 'run_crawl')
			self.framework.error('Pubmed is missed!', 'util/pubmed', 'run_crawl')
			return

		self._rawhtml += req.text

	@property
	def raw(self):
		return self._rawhtml

	@property
	def results(self):
		parser = self.framework.page_parse(self._rawhtml)
		results = parser.get_engine_results(self.xpaths, self.xpath_names)

		for count,article in enumerate(results):
			if count == self.max:
				break
			self._results.append({
				't': article['t'],
				'a': self.pubmed + article['a'],
				'c': article['c'],
				'd': article['d'] if len(article['d']) > 0 else 'No abstract available'
				})

		return self._results



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, q, count=10):
		""" wikipedia.org search 
			q          : Query for search
			count      : Number of results
		"""
		self.framework = main.framework
		self.q = q
		self.num = count
		self._links = []
		self._titles = []
		self._pids = []
		self.url = 'https://en.wikipedia.org/w/api.php'		
		self.headers = {'User-Agent' : 'OWASP Maryam(github.com/saeeddhqan/maryam)'}


	def run_crawl(self):
		self.framework.verbose('[WIKIPEDIA] Searching...', end='\r')
		payload = {
			'action': 'query',
			'list': 'search',
			'prop': '', 
			'srsearch': self.q, 
			'srlimit': self.num, 
			'utf8': '', 
			'format': 'json'
		}

		res = self.wiki_request(payload)
		if res is None:
			return 

		res = res.json()
		results = res['query']['search']

		for result in results:
			title = result['title']
			pid = result['pageid']
			link = 'https://en.wikipedia.org/wiki/' + '_'.join(title.split(' '))
			self._titles.append(title)
			self._links.append(link)
			self._pids.append(pid)


	def page(self):
		self.framework.verbose(f"[WIKIPEDIA] Getting page {self.q}...", end='\r')
		payload = {
			'action': 'query', 
			'pageids': self.q,
			'prop': 'info|extracts', 
			'inprop': 'url',  
			'explaintext': '',
			'exintro': '',
			'format': 'json'
		}

		res = self.wiki_request(payload)
		if not res:
			return

		res = res.json()
		return res['query']['pages'][str(self.q)]

	def wiki_request(self, payload):
		try:
			req = self.framework.request(
					url=self.url,
					params=payload,
					headers=self.headers,
					allow_redirects=False)
			return req
		except Exception as e:
			self.framework.error(f"ConnectionError: {e}", 'util/wikipedia', 'wiki_request')
			return None

	@property
	def links(self):
		return self._links

	@property
	def titles(self):
		return self._titles

	@property
	def pids(self):
		return self._pids

	@property
	def links_with_title(self):
		return zip(self._links, self._titles , self._pids)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from lxml import etree

class main:

	def __init__(self, q, limit=1):
		""" arxiv.org search engine

				q         : query for search
				limit     : maximum result count
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self._tree = ''
		self._rawxml = []
		self._articles = []
		self._links = []
		self._results = []
		self.url = 'https://export.arxiv.org/api/query'
		self.NSMAP = {'w3':'http://www.w3.org/2005/Atom'}

	def run_crawl(self):
		page = 1
		set_start = lambda x: (x - 1) * 10 + 1
		payload = {'search_query': f"all:{self.q}", 'start': set_start(page)}
		self.framework.verbose('[ARXIV] Searching the arxiv.org domain...')

		max_attempt = 0
		while True:
			try:
				req = self.framework.request(
						url=self.url,
						params=payload)
			except:
				self.framework.error('ConnectionError', 'arxiv', 'run_crawl')
				self.framework.error('ArXiv is missed!', 'arxiv', 'run_crawl')
				return
			else:
				self._rawxml.append(req.text)
				self._tree = etree.fromstring(req.text.encode('utf-8'))
				self._articles.extend(self.find(self._tree, './/w3:entry'))
				if page >= self.limit:
					break
				page += 1
				payload['start'] = set_start(page)

	def find(self, x, tofind):
		return x.findall(tofind, namespaces=self.NSMAP)

	@property
	def raw(self):
		return self._rawxml

	@property
	def articles(self):
		return self._articles

	@property
	def links(self):
		self._links = list(map(lambda x: x.text, 
			self.find(self._tree, './/w3:id')))
		return self._links

	@property
	def results(self):
		findlink = lambda x: self.find(x, './/w3:id')[0].text
		findauthors = lambda x: ', '.join(list(map(lambda x: x.text, 
				self.find(x, './/w3:author/w3:name'))))
		findtitle = lambda x: self.find(x, './/w3:title')[0].text
		findsummary = lambda x: self.find(x, './/w3:summary')[0].text.strip()

		for article in self._articles:
			self._results.append({
				't': findtitle(article),
				'a': findlink(article),
				'c': findauthors(article),
				'd': findsummary(article)
				})

		return self._results



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from re import search, I

class main:

	def __init__(self, content, headers):
		""" baidu.com search engine

			content	  : Web content
			headers	  : Web headers
		"""
		self.content = content
		self.headers = headers
		self._cms = None

	def run_crawl(self):
		for attr in dir(self):
			con1 = not attr.startswith('__')
			con2 = not attr.endswith('__')
			con3 = attr not in ('_cms', 'content', 'headers',
							 'cms', 'run_crawl', 'framework', 'initialize')
			if con1 and con2 and con3:
				getattr(self, attr)()
		
	def adobeaem(self):
		M = False
		M = search(
			r"<link[^>]*stylesheet[^>]*etc\/designs\/[^>]*\>[^<]*", self.content, I) is not None
		M |= search(
			r"<link[^>]*etc\/clientlibs\/[^>]*\>[^<]*", self.content, I) is not None
		M |= search(
			r"<script[^>]*etc\/clientlibs\/[^>]*\>[^<]*", self.content, I) is not None
		M |= search(
			r"<script[^>]*\/granite\/[^>]*(\.js\")+\>[^<]*", self.content, I) is not None
		if M:
			self._cms = 'Adobe AEM: Stack is based on Apache Sling + Apache Felix OSGi container + JCR Repo + Java'

	def drupal(self):
		M = False
		regexs = [r"\<script type\=\"text\/javascript\" src\=\"[^\"]*\/misc\/drupal.js[^\"]*\"\>\<\/script\>",
				  r"<[^>]+alt\=\"Powered by Drupal, an open source content management system\"",
				  r"@import \"[^\"]*\/misc\/drupal.css\"",
				  r"jQuery.extend\(drupal\.S*",
				  r"Drupal.extend\(\S*",
				  r"name=\"?generator\"? content=\"?Varbase",
				  r"jQuery.extend\(drupal\.S*",
				  r"data-drupal-selector=",
				  r"name=\"?generator\"? content=\"?Drupal ([0-9]+)",
				  ]
		if 'set-cookie' in self.headers.keys():
			if search(r"SESS[a-z0-9]{32}=[a-z0-9]{32}", self.headers["set-cookie"], I):
				M = "Drupal"
		if 'x-drupal-cache' in self.headers.keys():
			M = "Drupal"
		for i in regexs:
			tmp_M = search(i, self.content)
			if tmp_M:
				try:
					M = "Drupal version " + tmp_M.group(1)
				except IndexError:
					M = "Drupal"
				break
		if M:
			self._cms = M

	def joomla(self):
		M = False
		if 'set-cookie' in self.headers.keys():
			M |= search(
				r"mosvisitor=", self.headers["set-cookie"], I) is not None
		M |= search(
			r"\<meta name\=\"Generator\" content\=\"Joomla! - Copyright \(C\) 200[0-9] - 200[0-9] Open Source Matters. All rights reserved.\" \/\>", self.content, I) is not None
		M |= search(
			r"\<meta name\=\"generator\" content\=\"Joomla! (\d\.\d) - Open Source Content Management\" \/\>", self.content, I) is not None
		M |= search(
			r"Powered by \<a href\=\"http://www.joomla.org\"\>Joomla!\<\/a\>.", self.content, I) is not None
		if M:
			self._cms = 'Joomla'

	def magento(self):
		M = False
		if 'set-cookie' in self.headers.keys():
			M |= search(r"magento=[0-9a-f]+|frontend=[0-9a-z]+",
						   self.headers["set-cookie"], I) is not None
		M |= search(
			r"images/logo.gif\" alt\=\"Magento Commerce\" \/\>\<\/a\>\<\/h1\>", self.content, I) is not None
		M |= search(
			r"\<a href\=\"http://www.magentocommerce.com/bug-tracking\" id\=\"bug_tracking_link\"\>\<strong\>Report All Bugs\<\/strong\>\<\/a\>", self.content, I) is not None
		M |= search(
			r"\<link rel\=\"stylesheet\" type\=\"text/css\" href\=\"[^\"]+\/skin\/frontend\/[^\"]+\/css\/boxes.css\" media\=\"all\"", self.content, I) is not None
		M |= search(
			r"\<div id\=\"noscript-notice\" class\=\"magento-notice\"\>", self.content, I) is not None
		M |= search(
			r"Magento is a trademark of Magento Inc. Copyright &copy; ([0-9]{4}) Magento Inc", self.content, I) is not None
		if(M):
			self._cms = 'Magento'

	def plone(self):
		M = False
		if 'x-caching-rule-id' in self.headers.keys():
			M |= search(r"plone-content-types",
						   self.headers["x-caching-rule-id"], I) is not None
		if 'x-cache-rule' in self.headers.keys():
			M |= search(r"plone-content-types",
						   self.headers["x-cache-rule"], I) is not None
		M |= search(
			r"\<meta name\=\"generator\" content\=\"[^>]*http:\/\/plone.org\" \/>", self.content, I) is not None
		M |= search(
			r"(@import url|text\/css)[^>]*portal_css\/.*plone.*css(\)|\")", self.content, I) is not None
		M |= search(
			r"src\=\"[^\"]*ploneScripts[0-9]+.js\"", self.content, I) is not None
		M |= search(
			r"\<div class\=\"visualIcon contenttype-plone-site\"\>", self.content, I) is not None
		if(M):
			self._cms = "Plone"

	def silverstripe(self):
		M = False
		if 'set-cookie' in self.headers.keys():
			M |= search(
				r"PastVisitor=[0-9]+.*", self.headers["set-cookie"], I) is not None
		M |= search(
			r"\<meta name\=\"generator\"[^>]*content\=\"SilverStripe", self.content, I) is not None
		M |= search(
			r"\<link[^>]*stylesheet[^>]*layout.css[^>]*\>[^<]*\<link[^>]*stylesheet[^>]*typography.css[^>]*\>[^<]*\<link[^>]*stylesheet[^>]*form.css[^>]*\>", self.content, I) is not None
		M |= search(
			r"\<img src\=\"\/assets\/[^\/]+\/_resampled\/[^\"]+.jpg\"", self.content, I) is not None
		if(M):
			self._cms = "SilverStripe"

	def wordpress(self):
		M = False
		M |= search(
			r"\<meta name\=\"generator\" content\=\"WordPress.com\" \/\>", self.content, I) is not None
		M |= search(
			r"\<a href\=\"http://www.wordpress.com\"\>Powered by WordPress\<\/a\>", self.content, I) is not None
		M |= search(r"\<link rel\=\'https://api.w.org/\'",
					   self.content, I) is not None
		M |= search(r"\/wp-content\/plugins\/", self.content, I) is not None
		if M:
			self._cms = "WordPress"

	def concrete5(self):
		M = False
		M = search(
			r'<meta name="generator" content="concrete5',
			self.content) is not None
		M |= search(r'/packages/concrete5_theme/themes/',
					   self.content) is not None
		if M:
			self._cms = 'Concrete5'

	def typo3(self):
		M = False
		M = search(
			r'This website is powered by TYPO3|/typo3conf',
			self.content) is not None
		M |= search(
			r'"generator" content="TYPO3 CMS">|/typo3temp/assets/',
			self.content) is not None
		if M:
			self._cms = 'TYPO3'

	def hubspot(self):
		M = False
		M = search(
			r"<meta name=\"generator\" content=\"HubSpot\">", self.content) is not None
		M2 = search(
			r"<!-- Generated by the HubSpot Template Builder - template version ([0-9]{1,6}\.[0-9]{1,6}) -->", self.content)
		alert = "HubSpot"
		if M2:
			M |= True
			alert = f"HubSpot version {M2.group(1)}"
		if M:
			self._cms = alert

	def squarespace(self):
		M = False
		M = search(r"<!-- This is Squarespace. -->",
					  self.content) is not None
		if M:
			self._cms = 'Squarespace'
			
	def prestashop(self):
		M = False
		if 'set-cookie' in self.headers.keys():
			M |= search(r"PrestaShop-[a-z0-9A-Z]+", self.headers["set-cookie"], I) is not None
		if 'Powered-By' in self.headers.keys():
			M |= search(r"^Prestashop$",self.headers["Powered-By"], I) is not None
		M |= search(r"<a.*Ecommerce software by PrestaShop™[\s]*</a>", self.content) is not None
		if M:
			self._cms = 'Prestashop'

	@property
	def cms(self):
		return self._cms



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Documentations Search',
	'author': 'Saeed',
	'version': '0.8',
	'description': 'Search in open-sources to find relevant documents. filetypes[pdf,doc,docx,ppt,pptx,xlsx,txt,..].',
	'sources': ('bing', 'google', 'yahoo', 'metacrawler', 'ask',
		'startpage', 'exalead', 'carrot2', 'qwant', 'millionshort', 'duckduckgo', 'gigablast', 'dogpile'),
	'options': (
		('query', None, True, 'Host Name, Company Name, keyword, query, etc', '-q', 'store', str),
		('file', 'pdf', True, 'Filetype [pdf,doc,docx,ppt,pptx,xlsx,txt,..]', '-f', 'store', str),
		('limit', 2, False, 'Search limit(number of pages, default=2)', '-l', 'store', int),
		('count', 50, False, 'number of results per page(min=10)', '-c', 'store', int),
		('site', False, False, 'If this is set, search just limited to the site', '-s', 'store_false', bool),
		('engines', 'exalead,bing,google', True, 'Search engines with comma separator', '-e', 'store', str),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
	),
	'examples': ('docs_search -q amazon -f pdf -e google,bing,metacrawler --thread 3',
		'docs_search -q amazon -f pdf -e google,bing,metacrawler -l 3')
}
	
DOCS = []

def search(self, name, q, q_formats, limit, count):
	global DOCS
	try:
		engine = getattr(self, name)
		q = q
		varnames = engine.__init__.__code__.co_varnames
		if 'limit' in varnames and 'count' in varnames:
			attr = engine(q, limit, count)
		elif 'limit' in varnames:
			attr = engine(q, limit)
		else:
			attr = engine(q)

		attr.run_crawl()
		DOCS += attr.docs
	except Exception as e:
		print(e)

def module_api(self):
	query = self.options['query']
	_type = self.options['file'].lower()
	limit = self.options['limit']
	count = self.options['count']
	engines = self.options['engines'].lower().split(',')
	# Make dork
	if self.options['site']:
		dork = self.urlib(f"filetype:{_type} site:{self.options['site']} {query}").quote
	else:
		dork = f"{query} filetype:{_type}"

	self.thread(search, self.options['thread'], engines, dork, {}, limit, count, meta['sources'])
	output = {_type : list(set(DOCS))}
	self.save_gather(output, 'osint/docs_search', query, \
		[_type], output=self.options['output'])
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


class main:
	# framework = None
	def __init__(self, q, limit=1, count=10):
		""" google.com search engine
			q     : Query for search
			limit : Number of pages
			count : Number of results
		"""
		self.framework = main.framework
		self.q = q
		self.agent = 'Mozilla/5.0 (X11; Linux x86_64; rv:86.0) Gecko/20100101 Firefox/86.0'
		self.xpath_name_original = {
			'results': '//div[@class="g"]|//div[@class="g tF2Cxc"]',
			'results_content': './/div[@class="IsZvec"]',
			'results_title': './/h3[1]',
			'results_a': './/div[@class="yuRUbf"]/a',
			'results_cite': './/div[@class="yuRUbf"]/a//cite'
		}
		self.xpath_original = {
			self.xpath_name_original['results']: [
				self.xpath_name_original['results_content'],
				self.xpath_name_original['results_title'],
				self.xpath_name_original['results_a'],
				self.xpath_name_original['results_cite']
			]
		}
		self.url = 'https://www.google.com/search'
		self._pages = ''
		self._first_page = ''
		self.limit = limit + 1
		self.count = count

	def run_crawl(self):
		page = 1
		set_page = lambda x: (x - 1) * self.count
		payload = {'num': self.count, 'start': set_page(page), 'ie': 'utf-8', 'oe': 'utf-8', 'q': self.q, 'filter': '0'}
		max_attempt = 0
		while True:
			self.framework.verbose(f"[GOOGLE] Searching in {page} page...", end='\r')
			try:
				req = self.framework.request(
					url=self.url,
					params=payload,
					headers={'user-agent': self.agent},
					allow_redirects=True)
			except Exception as e:
				self.framework.error(f"ConnectionError: {e}", 'util/google', 'run_crawl')
				max_attempt += 1
				if max_attempt == self.limit:
					self.framework.error('Google is missed!', 'util/goolge', 'run_crawl')
					break
			else:
				if req.status_code in (503, 429):
					self.framework.error('Google CAPTCHA triggered.', 'util/google', 'run_crawl')
					break

				if req.status_code in (301, 302):
					redirect = req.headers['location']
					req = self.framework.request(url=redirect, allow_redirects=False)

				self._pages += req.text
				if page == 1:
					self._first_page += req.text
				page += 1
				payload['start'] = set_page(page)
				if page >= self.limit:
					break
	@property
	def google_card(self):
		card_xpath_name = {
			'card': '//div[@id="wp-tabs-container"]',
			'card_content': './/div[@class="kno-rdesc"]',
			'card_info': './/div[@class="rVusze"]'
		}
		xpath = {
			card_xpath_name['card']: [
				card_xpath_name['card_content'],
				card_xpath_name['card_info']
			]
		}
		card_xpath_second = {
			'card': '//div[@class="SzZmKb"]',
			'card_img': './/g-img/@data-lpage',
			'card_name': './/h2[@class="qrShPb kno-ecr-pt PZPZlf mfMhoc"]',
			'card_known_as': './/div[@data-attrid="subtitle"]'
		}
		xpath2 = {
			card_xpath_second['card']: [
				card_xpath_second['card_img'],
				card_xpath_second['card_name'],
				card_xpath_second['card_known_as']
			]
		}
		card_xpath_social = {
			'card': '//g-link[@class="fl"]',
			'card_href': './/a',
		}
		xpath3 = {
			card_xpath_social['card']: [
				card_xpath_social['card_href']
			]
		}
		parser = self.framework.page_parse(self._first_page)
		xpath_results = parser.html_fromstring(xpath)
		xpath_results2 = parser.html_fromstring(xpath2)
		xpath_results3 = parser.html_fromstring(xpath3)
		output = {'content': '', 'info': []}
		root = xpath_results[card_xpath_name['card']]
		root2 = xpath_results2[card_xpath_second['card']]
		root3 = xpath_results3[card_xpath_social['card']]
		if root[card_xpath_name['card_content']]:
			output['content'] = root[card_xpath_name['card_content']][0].text_content()
		else:
			card_xpath_name = {
				'card': '//div[@class="LuVEUc B03h3d P6OZi V14nKc ptcLIOszQJu__wholepage-card wp-ms"]',
				'card_content': './/div[@class="kno-rdesc"]',
				'card_info': './/div[@class="rVusze"]'
			}
			xpath = {
				card_xpath_name['card']: [
					card_xpath_name['card_content'],
					card_xpath_name['card_info']
				]
			}
			xpath_results = parser.html_fromstring(xpath)
			root = xpath_results[card_xpath_name['card']]
			if root[card_xpath_name['card_content']]:
				output['content'] = root[card_xpath_name['card_content']][0].text_content()
			else:
				output['content'] = ''
		img = root2[card_xpath_second['card_img']]
		name = root2[card_xpath_second['card_name']]
		known_as = root2[card_xpath_second['card_known_as']]
		if img:
			output['img'] = img[0]
		if name:
			output['name'] = name[0].text_content()
		if known_as:
			output['known_as'] = known_as[0].text_content()
		for piece in root[card_xpath_name['card_info']]:
			output['info'].append(piece.text_content())
		social = root3[card_xpath_social['card_href']]
		output['social'] = []
		for piece in social:
			output['social'].append(piece.get('href'))
		return output

	@property
	def results(self):
		parser = self.framework.page_parse(self._pages)
		results = parser.get_engine_results(self.xpath_original, self.xpath_name_original)
		return results

	@property
	def pages(self):
		return self._pages

	@property
	def links(self):
		links = [x['a'] for x in self.results]
		return links

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q, self.links)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q, limit=2):
		""" netcraft.com for find dns

			q 		  : Query for search
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self.base_url = f"https://www.virustotal.com/ui/domains/{q}/subdomains?relationships=resolutions&cursor=STMwCi4=&limit=40"
		self._pages = ''

	def request(self, url):
		try:
			req = self.framework.request(url=url)
		except:
			self.framework.error('[VIRUSTOTAL] ConnectionError')
			self.framework.error('VirusTotal is missed!')
			return False
		return req

	def run_crawl(self):
		count = 0
		while True:
			self.framework.verbose(f"[VIRUSTOTAL] Searching in {count} page...")
			req = self.request(self.base_url)
			if not req:
				return
			self._pages += req.text
			json = req.json()
			if count+1 == self.limit:
				break
			if 'links' in json:
				if 'next' in json['links']:
					self.base_url = json['links']['next']
				else:
					return
			else:
				return
			count += 1

	@property
	def pages(self):
		return self._pages

	@property
	def dns(self):
		resp = []
		parser = self.framework.page_parse(str(self._pages)).get_dns(self.q)
		for host in list(set(parser)):
			if host[0].isdigit():
				matches = re.match(r".+([0-9])[^0-9]*$", host)
				host = host[matches.start(1) + 1:]
			if host.startswith('.'):
				host = host[1:]
			resp.append(host)
		return resp



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import re

meta = {
        'name': 'Gigablast',
        'author': 'Kaushik',
        'version': '0.1',
        'description': 'Gigablast provides large-scale, high-performance, \
			real-time information retrieval technology and services for partner sites',
        'sources': ('gigablast',),
        'options': (
                ('query', None, True, 'Query to search', '-q', 'store', str),
		('limit', 15, False, 'Max result count (default=15)', '-l', 'store', int),
        ),
        'examples': ('gigablast -q <QUERY> -l 20',)
}

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	run = self.gigablast(query, limit)
	run.run_crawl()
	output = {'results': run.results}

	self.save_gather(output, 'search/gigablast', query, output=self.options['output'])
	return output

def module_run(self):
	self.search_engine_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import concurrent.futures
from socket import gethostbyname

from maryam.core.basedir import BASEDIR

meta = {
	'name': 'DNS Brute Force',
	'author': 'Saeed',
	'version': '1.1',
	'description': 'TLD brute force attack, supports cocurrency.',
	'comments': ('wordlist option can be an url',),
	'options': (
		('domain', None, True, 'Domain name without https?://', '-d', 'store', str),
		('count', None, False, 'Number of payloads len(max=count of payloads). default is max',
							 '-c', 'store', int),
		('wordlist', os.path.join(BASEDIR, 'data', 'tlds.txt'), False, 
							'wordlist address. default is dnsnames.txt in data folder', '-w', 'store', str),
		('thread', 8, False, 'The number of links that open per round(default=8)', '-t', 'store', int),
		('ips', False, False, 'Show ip addresses', '-i', 'store_true', bool),
	),
	'examples': ('tldbrute -d <DOMAIN> --output',
				'tldbrute -d <DOMAIN> -w <WORDLIST>')
}

HOSTNAMES = []

def thread(self, function, hostname, wordlist, thread_count):
	threadpool = concurrent.futures.ThreadPoolExecutor(max_workers=thread_count)
	futures = (threadpool.submit(function, self, hostname, word) for word in wordlist if not word.startswith('#'))
	counter = 1
	for _ in concurrent.futures.as_completed(futures):
		print(f"Checking payload {counter}, hits: {len(HOSTNAMES)}", end='\r')
		counter += 1
	print(os.linesep)

def request_checker(self, hostname, word):
	hostname = f"{hostname}.{word}".lower()
	try:
		req = self.request(hostname)
	except Exception as e:
		if 'timed out' in str(e.args):
			self.verbose(f"{hostname} => Timed out", 'O')
		else:
			self.debug(f"{list(e.args)[0]}:'{hostname}'", 'O')
	else:
		if self.options['ips']:
			try:
				ip = gethostbyname(hostname)
			except:
				ip = '-'
			hostname = f"{hostname} : {ip}"
		HOSTNAMES.append(hostname)
		self.output(f"{hostname}{' '*50}", 'G')

def module_api(self):
	domain = self.options['domain']
	urlib = self.urlib(domain)
	domain = urlib.sub_service('http')
	hostname = urlib.netroot
	hostname = hostname.split('.')[0]
	wordlist = self.options['wordlist']
	count = self.options['count']

	dlist = self._is_readable(wordlist)
	if dlist:
		dlist = dlist.read().split()
	else:
		dlist = []

	dlen = len(dlist)
	if not count:
		count = dlen
	else:
		count = dlen if count > dlen else count

	self.heading(f"Starting TLD brute force with {count} payload", level=0)
	thread(self, request_checker, hostname, dlist[:count], self.options['thread'])
	output = {'hostnames': HOSTNAMES}

	self.save_gather(output, 'footprint/tldbrute', domain, output=self.options['output'])
	return output

def module_run(self):
	module_api(self)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Wikileaks Search',
	'author': 'Aman Singh',
	'version': '0.1',
	'description': 'WikiLeaks is an international non-profit organisation that publishes news leaks and classified media provided by anonymous sources.',
	'sources': ('wikileaks',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
	),
	'examples': ('wikileaks -q <QUERY> -l 15 --output',)
}

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	run = self.wikileaks(query, limit)
	run.run_crawl()

	output = {'results': []}
	links = run.links_with_data

	for item in links:
		output['results'].append(item)

	self.save_gather(output, 'search/wikileaks', query, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)
	for result in output['results']:
		self.output(result['title'])
		self.output(f"\t{result['link']}",'G')




"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import re
import json

class main:

		def __init__(self, q, limit=15):
			""" Gigablast search engine
					q     : Query to search
					limit : Number of pages
			"""
			self.framework = main.framework
			self.q = q
			self._max = limit
			self._json = ''
			self._results = []

		def run_crawl(self):
			from bs4 import BeautifulSoup as bs

			self.q = self.framework.urlib(self.q).quote
			self.framework.verbose('Searching Gigablast...')
			first_url = 'https://www.gigablast.com/search'
			payload = {'q': self.q, 'format': 'json'}
			try:
				req = self.framework.request(url=first_url,
						params=payload)
			except:
				self.framework.error('ConnectionError.', 'util/gigablast', 'run_crawl')
				self.framework.error('Gigablast is missed!', 'util/gigablast', 'run_crawl')
			else:
				soup = bs(req.text,'html.parser')
				script = soup.find('body')['onload']
				params1 = re.findall(r"var uxrl='/search(.*?)'", script)[0]
				params2 = re.findall(r"uxrl=uxrl\+'(.*?)'", script)[0]
				final_url = first_url + params1 + params2
				try:
					req = self.framework.request(url=final_url)
				except:
					self.framework.error('ConnectionError.', 'util/gigablast', 'run_crawl')
					self.framework.error('Gigablast is missed!', 'util/gigablast', 'run_crawl')
				else:
					self._json = req.json()['results']

		@property
		def json(self):
			return self._json

		@property
		def raw(self):
			return json.dumps(self._json)

		@property
		def dns(self):
			return self.framework.page_parse(self.raw).get_dns(self.q)

		@property
		def emails(self):
			return self.framework.page_parse(self.raw).get_emails(self.q)

		@property
		def docs(self):
			return self.framework.page_parse(self.raw).get_docs(self.q)

		@property
		def results(self):
			for count, result in enumerate(self._json):
				if count >= self._max:
					break
				urlib = self.framework.urlib(result['url'])
				a = urlib.unquote
				cite = result.get('subTitle') or self.framework.meta_search_util().make_cite(a)
				self._results.append({
					't': result['title'],
					'a': a,
					'c': cite,
					'd': result['sum'],
					})
			return self._results



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

meta = {
	'name': 'Instagram Search',
	'author': 'Aman Singh',
	'version': '0.7',
	'description': 'Search your query in the Instagram and show the results.',
	'sources': ('google', 'carrot2', 'bing', 'yahoo', 'millionshort', 'qwant', 'duckduckgo', 'instagram'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of links per page(min=10, max=100, default=50)', '-c', 'store', int),
		('engine', 'google', False, 'Engine names for search(default=google, instagram)', '-e', 'store', str),
		('session_id', None, False, 'Insta Account Session_id for more details results (default="")', '-s', 'store', str),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
	),
    'examples': ('instagram -q <QUERY> -l 150 -s <session> --output',)
}

LINKS = []
PAGES = ''
USERDATA = {}
FOLLOWERS = []
FOLLOWING = []
POST = []

def search(self, name, q, q_formats, limit, count, session_id):
	global PAGES,LINKS,USERDATA,FOLLOWERS,FOLLOWING,POST
	eng = name
	query = q
	engine = getattr(self, name)
	q = q_formats[f"{name}_q"] if f"{name}_q" in q_formats else q_formats['default_q']
	varnames = engine.__init__.__code__.co_varnames

	# for instagram
	if eng == 'instagram':
		attr = self.instagram(query, limit=limit, session_id=session_id)
		attr.run_crawl()
		USERDATA = attr.userdata
		FOLLOWERS = attr.followers
		FOLLOWING = attr.following
		POST = attr.post
		USERDATA = attr.userdata
	# for others
	else :
		if 'limit' in varnames and 'count' in varnames:
			attr = engine(q, limit, count)
		elif 'limit' in varnames:
			attr = engine(q, limit)
		else:
			attr = engine(q)
		# append data to return here
		attr.run_crawl()
		LINKS += attr.links
		PAGES += attr.pages

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	session_id = self.options['session_id'] or ''
	engine = self.options['engine'].split(',')
	output = {
		'people': [],
		'posts': [],
		'hashtags': [],
		'followers': [],
		'following': [],
		'userdata': {},
		}
	if 'instagram' in engine and not session_id:
		msg = '"sessionid" field from cookies is required for instagram engine.'
		self.error(msg, 'instagram', 'module_api')
		return

	q_formats = {
		'default_q': f"site:www.instagram.com {query}",
		'google_q': f"site:www.instagram.com inurl:{query}",
		'instagram': f"{query}"
	}

	self.thread(search, self.options['thread'], engine, query, q_formats, limit, count, session_id, meta['sources'])

	# for other modules
	usernames = self.page_parse(PAGES).get_networks
	for _id in list(set(usernames.get('Instagram'))):
		if _id[-2:] == '/p' or _id[-8:] == '/explore':
			continue
		_id = f"{_id[_id.find('/')+1:]}"
		if _id not in output['people']:
			output['people'].append(_id)

	links = list(self.reglib().filter(r"https?://(www\.)?instagram\.com/", list(set(LINKS))))
	for link in self.reglib().filter(lambda x: '/explore/tags/' in x, links):
		tag = re.sub(r'https?://(www\.)?instagram\.com/explore/tags/', '', link)
		if re.search(r'"[\w\d_\-\/]+$"', tag):
			tag = tag.rsplit('/')
			output['hashtags'].append(tag[0])

	post_data_extracted= self.reglib().filter(r"https?://(www\.)?instagram\.com/p/[\w_\-0-9]+/", links)
	for link in post_data_extracted:
		output['posts'].append(link)

	# for insta
	if USERDATA:
		for key in USERDATA:
			if isinstance(USERDATA[key], str) and '\n' in USERDATA[key]: 
				USERDATA[key] = USERDATA[key].encode('utf-8')
			output['userdata'][key] = USERDATA[key]

	output['posts'].extend(POST) 
	output['followers'] = FOLLOWERS
	output['following']= FOLLOWING
	output['links'] = links

	output = {key: val for key, val in output.items() if val} 

	self.save_gather(output,'search/instagram', query, output=self.options.get('output'))
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import webbrowser

class main:

	def __init__(self, q, limit=2, count=100):
		""" yandex.com search engine

			q 		  : query for search
			limit	  : Number of pages
			count	  : Number of links
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote
		self.count = count
		self.limit = limit
		self._pages = ''
		self.yandex = 'yandex.com'

	def run_crawl(self):
		urls = [f"https://{self.yandex}/search/?text={self.q}&msid=1653298889620368-11933310814821699217-sas3-0816-dd1-sas-l7-balancer-8080-BAL-4845&suggest_reqid=492440725165329881894805002353660&numdoc={self.count}&p={i}" for i in range(1, self.limit+1)]
		max_attempt = len(urls)
		for url in range(len(urls)):
			self.framework.debug(f"[YANDEX] Searching in {url} page...")
			print(urls[url])
			try:
				req = self.framework.request(url=urls[url], allow_redirects=True)
			except:
				self.framework.error('ConnectionError', 'util/yandex', 'run_crawl')
				max_attempt -= 1
				if max_attempt == 0:
					self.framework.error('Yandex is missed!', 'util/yandex', 'run_crawl')
					break
			else:
				if '<title>Oops!</title>' in req.text:
					self.framework.error('Yandex CAPTCHA triggered.', 'util/yandex', 'run_crawl')
					return

				page = req.text
				if ']">next</a>' not in page:
					self._pages += page
					break
				self._pages += page

	@property
	def pages(self):
		return self._pages

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import concurrent.futures

meta = {
	'name': 'Find Interest Files',
	'author': 'Saeed',
	'version': '0.9',
	'description': 'Search hosts for interesting files.',
	'options': (
		('domain', None, True, 'Domain name', '-d', 'store', str),
		('logs', False, False, 'Log files search(7 payload)', '--logs', 'store_true', bool),
		('backup', False, False, 'Backup files search(148 payload)', '--backup', 'store_true', bool),
		('apache', False, False, 'Apache status check(28 payload', '--apache', 'store_true', bool),
		('admin', False, False, 'Admin panel check(482 payload)', '--admin', 'store_true', bool),
		('soap', False, False, 
			'SOAP and REST-based web services(3 payload). it will be helpful in attacking both SOAP and REST-based web services', 
			'--soap', 'store_true', bool),
		('thread', 8, False, 'The number of links that open per round(default=8)', '-t', 'store', int),
	),
	'examples': ('interest_files -d <DOMAIN> --logs --admin',
		'interest_files -d <DOMAIN> --backup --apache --admin --output')
}

OUTPUT = {}

def thread(self, function, hostname, wordlist,\
			thread_count, method, header=(), content=None,\
			status_codes=None, not_status_codes=None):
	if content is None:
		content = []
	if status_codes is None:
		status_codes = []
	if not_status_codes is None:
		not_status_codes = []
	threadpool = concurrent.futures.ThreadPoolExecutor(max_workers=thread_count)
	futures = (threadpool.submit(function, self, hostname, word, method, header,\
				content, status_codes, not_status_codes) for word in wordlist if not '#' in word)
	counter = 1
	for x in concurrent.futures.as_completed(futures):
		print(f"Checking payload {counter}, hits: {len(OUTPUT[method])}", end='\r')
		counter += 1
	print('')

def default(self, hostname, item, method, header, content, status_codes, not_status_codes):
	if isinstance(item, tuple):
		content = [item[1]]
		urjoin = self.urlib(hostname).join(item[0])
	else:
		urjoin = self.urlib(hostname).join(item)

	try:
		req = self.request(urjoin)
	except Exception as e:
		if 'timed out' in str(e.args):
			self.verbose(f"{urjoin} => Timed out", 'O')
		else:
			self.debug(f"{list(e.args)[0]}:'{urjoin}'", 'O')
	else:
		if 'Location' in req.headers:
			location = req.headers['Location']
			try:
				req = self.request(location)
			except:
				return
		conds = []
		if header:
			conds.append(req.headers[header[0]] == header[1])
		if content:
			conds.append(all([x in req.text for x in content]))
		if status_codes:
			conds.append(req.status_code in status_codes)
		if not_status_codes:
			conds.append(req.status_code not in not_status_codes)
		if all(conds):
			self.output(f"{urjoin}{' ' * 50}", 'G')
			OUTPUT[method].append(urjoin)
		else:
			self.debug(f"{urjoin} > not found{' '*50}", 'O')

def module_api(self):
	url = self.options['domain']
	thread_count = self.options['thread']

	OUTPUT['common_files'] = []
	tops = {
		'web-console': 'Administration',
		'elmah.axd': 'Error Log fo',
		'sitemap.xml': '<?xml',
		'sitemap.xml.gz': '<?xml',
		'robots.txt': 'User-agent:',
		'server-status': '>Apache Status<',
		'jmx-console': 'JBoss',
		'admin-console': 'index.seam'}

	thread(self, default, url, list(tops.items()), \
		thread_count, 'common_files', not_status_codes=[301, 404])

	OUTPUT['phpinfo'] = []
	php_info_list = ['INSTALL.php?mode=phpinfo', 'PhpInfo.php', 'PHPinfo.php', 'phpinfo.php', 'PHPINFO.php', 'phpInfo.php', 'info.php', 'Info.php', 
					 'INFO.php', 'test.php?mode=phpinfo', 'index.php?view=phpinfo', 'index.php?mode=phpinfo', 'TEST.php?mode=phpinfo',
					 'install.php?mode=phpinfo', 'admin.php?mode=phpinfo', 'phpversion.php', 'phpVersion.php', 'test1.php', 'test.php', 'test2.php',
					 'phpinfo1.php', 'phpInfo1.php', 'info1.php', 'PHPversion.php', 'test', 'php', 'a.php', 'x.php', 'xx.php', 'xxx.php']
	
	thread(self, default, url, php_info_list, \
		thread_count, 'phpinfo', status_codes=[200, 201, 204, 202, 206])

	### LOGS Files ####
	if self.options['logs']:
		OUTPUT['logs'] = []
		log_payloads = ['error_log', 'error.log', 'debug.log', 'php.errors',
				 'php5-fpm.log', 'php_errors.log', 'security.txt']

		thread(self, default, url, log_payloads, thread_count, \
			'logs', status_codes=[200, 201, 204, 202, 206], header=('Content-Type', 'text/plain'))

	## BACKUP Brute Force ##
	if self.options['backup']:
		OUTPUT['backup'] = []
		backup_payloads = ['1.txt', '2.txt', '1.gz', '1.rar', '1.save', '1.tar', '1.tar.bz2', '1.tar.gz', '1.tgz', '1.tmp', '1.zip', '2.back', '2.backup', '2.gz', '2.rar', '2.save', '2.tar', '2.tar.bz2', '2.tar.gz', '2.tgz', '2.tmp', '2.zip', 'backup.back', 'backup.backup', 'backup.bak', 'backup.bck', 'backup.bkp', 'backup.copy', 'backup.gz', 'backup.old', 'backup.orig', 'backup.rar', 'backup.sav', 'backup.save', 'backup.sql.back', 'backup.sql.backup', 'backup.sql.bak', 'backup.sql.bck', 'backup.sql.bkp', 'backup.sql.copy', 'backup.sql.gz', 'backup.sql.old', 'backup.sql.orig', 'backup.sql.rar', 'backup.sql.sav', 'backup.sql.save', 'backup.sql.tar', 'backup.sql.tar.bz2', 'backup.sql.tar.gz', 'backup.sql.tgz', 'backup.sql.tmp', 'backup.sql.txt', 'backup.sql.zip', 'backup.tar', 'backup.tar.bz2', 'backup.tar.gz', 'backup.tgz', 'backup.txt', 'backup.zip', 'database.back', 'database.backup', 'database.bak', 'database.bck', 'database.bkp', 'database.copy', 'database.gz', 'database.old', 'database.orig', 'database.rar', 'database.sav', 'database.save', 'database.sql~', 'database.sql.back', 'database.sql.backup', 'database.sql.bak',
				 'database.sql.bck', 'database.sql.bkp', '~backup', 'backup', 'database.sql.copy', 'database.sql.gz', 'database.sql.old', 'database.sql.orig', 'database.sql.rar', 'database.sql.sav', 'database.sql.save', 'database.sql.tar', 'database.sql.tar.bz2', 'database.sql.tar.gz', 'database.sql.tgz', 'database.sql.tmp', 'database.sql.txt', 'database.sql.zip', 'database.tar', 'database.tar.bz2', 'database.tar.gz', 'database.tgz', 'database.tmp', 'database.txt', 'database.zip', 'site.back', 'site.backup', 'site.bak', 'site.bck', 'site.bkp', 'site.copy', 'site.gz', 'site.old', 'site.orig', 'site.rar', 'site.sav', 'site.save', 'site.tar', 'site.tar.bz2', 'site.tar.gz', 'site.tgz', 'site.zip', 'sql.zip.back', 'sql.zip.backup', 'sql.zip.bak', 'sql.zip.bck', 'sql.zip.bkp', 'sql.zip.copy', 'sql.zip.gz', 'sql.zip.old', 'sql.zip.orig', 'sql.zip.save', 'sql.zip.tar', 'sql.zip.tar.bz2', 'sql.zip.tar.gz', 'sql.zip.tgz', 'upload.back', 'upload.backup', 'upload.bak', 'upload.bck', 'upload.bkp', 'upload.copy', 'upload.gz', 'upload.old', 'upload.orig', 'upload.rar', 'upload.sav', 'upload.save', 'upload.tar', 'upload.tar.bz2', 'upload.tar.gz', 'upload.tgz', 'upload.zip']

		thread(self, default, url, backup_payloads, thread_count, \
			'backup', status_codes=[200, 201, 204, 202, 206], header=('Content-Type', 'text/plain'))
		
	## APACHE Brute Force ##
	if self.options['apache']:
		OUTPUT['apache'] = []
		apache_payloads = ['perl-status', 'server-status', 'server-info',
				 'stronghold-info', 'stronghold-status', '.htaccess',
				 '.htpasswd', '.meta', '.web', 'access_log', 'cgi', 'cgi-bin', 'cgi-pub', 'cgi-script',
				 'dummy', 'error', 'htdocs', 'httpd', 'httpd.pid', 'icons', 'logs', 'manual', 'phf',
				 'printenv', 'status', 'test-cgi', 'tmp', 'php.ini']

		thread(self, default, url, apache_payloads, thread_count, \
			'apache', status_codes=[200, 201, 204, 202, 206])

	## SOAP Brute Force ##
	if self.options['soap']:
		self.verbose('[SOAP] ')
		OUTPUT['soap'] = []
		soap_payloads = ['GetAccount', 'GetUser', 'GetCCN']

		thread(self, default, url, soap_payloads, thread_count, \
			'soap', status_codes=[200, 201, 204, 202, 206])

	# ## ADMIN Brute Force ###
	if self.options['admin']:
		OUTPUT['admin'] = []
		admin_payloads = ['~adm', '~admin', '~administrator', '~amanda', '~apache', '~bin', '~ftp', '~guest', '~http', '~httpd', '~log', '~logs', '~lp', '~mail', '~nobody', '~operator', '~root', '~sys', '~sysadm', '~sysadmin', '~test', '~tmp', '~user', '~webmaster', '~www', 'wp-admin', 'wp-login.php', 'administrator', 'database.sql', 'backup-db', 'mysql.sql', 'phpmyadmin', 'server-status', 'server-info', 'php.php', 'test.php', '.git', '.htaccess.old', '.htaccess.save', '.htaccess.txt', '.php-ini', 'php-ini', 'FCKeditor', 'FCK', 'editor', 'Desktop.ini', 'INSTALL', 'install', 'install.php', 'update', 'upgrade', 'upgrade.php', 'update.php', 'LICENSE', 'LICENSE.txt', 'Server.php', 'WS_FTP.LOG', 'WS_FTP.ini', 'WS_FTP.log', 'Web.config', 'Webalizer', 'webalizer', 'config.php', 'config.php.new', 'config.php~', 'controlpanel', 'cpanel', 'favicon.ico', 'old', 'php-error', 'php.ini~', 'php.ini', 'php.log', 'robots.txt', 'security', 'webdav', '1', 'acceso.asp', 'acceso.php', 'access', 'access.php', 'account', 'account.asp', 'account.html', 'account.php', 'acct_login', '_adm_', '_adm', 'adm', 'adm2', 'adm/admloginuser.asp', 'adm/admloginuser.php', 'adm.asp', 'adm_auth.asp', 'adm_auth.php', 'adm.html', '_admin_', '_admin', 'admin1', 'admin1.asp', 'admin1.html', 'admin1.php', 'admin2', 'admin2.asp', 'admin2.html', 'admin2/index', 'admin2/index.asp', 'admin2/index.php', 'admin2/login.asp', 'admin2/login.php', 'admin2.php', 'admin3', 'admin4', 'admin4_account', 'admin4_colon', 'admin5', 'admin/account.asp', 'admin/account.html', 'admin/account.php', 'admin/add_banner.php', 'addblog.php', 'admin/add_gallery_image.php', 'admin/add.php', 'admin/add', 'room.php', 'slider.php', 'add_testimonials.php', 'admin/admin', 'admin/adminarea.php', 'admin/admin.asp', 'admin/AdminDashboard.php', 'home.php', 'admin/AdminHome.php', 'admin/admin.html', 'admin/admin_index.php', 'admin/admin_login.asp', 'login.asp', 'admin/adminLogin.asp', 'admin/admin_login.html', 'login.html', 'admin/adminLogin.html', 'admin/admin_login.php', 'adminLogin.php', 'admin/admin_management.php', 'admin/admin.php', 'admin/admin_users.php', 'admin/adminview.php', 'admin/adm.php', 'admin_area', 'adminarea', 'admin_area/admin.asp', 'adminarea/admin.asp', 
				'admin_area/admin.html', 'adminarea/admin.html', 'admin.php', 'adminarea/admin.php', 'admin_area/index.asp', 'adminarea/index.asp', 'admin_area/index.html', 'adminarea/index.html', 'admin_area/index.php', 'adminarea/index.php', 'admin_area/login.asp', 'adminarea/login.asp', 'admin_area/login.html', 'adminarea/login.html', 'admin_area/login.php', 'adminarea/login.php', 'admin.asp', 'admin/banner.php', 'admin/banners_report.php', 'admin/category.php', 'change_gallery.php', 'admin/checklogin.php', 'admin/configration.php', 'admincontrol.asp', 'admincontrol.html', 'admincontrol/login.asp', 'admincontrol/login.html', 'admincontrol/login.php', 'admin/control_pages/admin_home.php', 'admin/controlpanel.asp', 'admin/controlpanel.html', 'admin/controlpanel.php', 'admincontrol.php', 'admin/cpanel.php', 'admin/cp.asp', 'admin/CPhome.php', 'admin/cp.html', 'admincp/index.asp', 'admincp/index.html', 'admincp/login.asp', 'admin/cp.php', 'admin/dashboard/index.php', 'admin/dashboard.php', 'admin/dashbord.php', 'admin/dash.php', 'admin/default.php', 'adm/index.asp', 'adm/index.html', 'adm/index.php', 'admin/enter.php', 'admin/event.php', 'admin/form.php', 'admin/gallery.php', 'admin/headline.php', 'admin/home.asp', 'admin/home.html', 'admin_home.php', 'admin/home.php', 'admin.html', 'admin/index.asp', 'admin/index', 'digital.php', 'index.html', 'admin/index.php', 'admin/index_ref.php', 'admin/initialadmin.php', 'administer', 'administr8', 'administr8.asp', 'administr8.html', 'administr8.php', 'administracion.php', 'administrador', 'administratie', 'administration', 'administration.html', 'administration.php', '_administrator_', '_administrator', 'administrator/account.asp', 'administrator/account.html', 'administrator/account.php', 'administratoraccounts', 'administrator.asp', 'administrator.html', 'administrator/index.asp', 'administrator/index.html',
				'administratorlogin', 'administrator/login.asp', 'administratorlogin.asp', 'administrator/login.html', 'administrator/login.php', 'administratorlogin.php', 'administrator.php', 'administrators', 'administrivia', 'leads.php', 'admin/list_gallery.php', 'admin/login', 'adminLogin', 'admin_login.asp', 'admin/login.asp', 'adminLogin.asp', 'admin_login.html', 'admin/login.html', 'adminLogin.html', 'ADMIN/login.html', 'admin_login.php', 'login.php ', 'admin', 'admin/login.php', 'ADMIN/login.php', 'admin/login_success.php', 'admin/loginsuccess.php', 'admin/log.php', 'admin_main.html', 'admin/main_page.php', 'admin/main.php', 'ManageAdmin.php', 'admin/manageImages.php', 'admin/manage_team.php', 'admin/member_home.php', 'admin/moderator.php', 'admin/my_account.php', 'admin/myaccount.php', 'admin/overview.php', 'admin/page_management.php', 'admin/pages/home_admin.php', 'adminpanel', 'adminpanel.asp', 'adminpanel.html', 'adminpanel.php', 'Admin/private', 'adminpro', 'admin/product.php', 'admin/products.php', 'admins', 'admins.asp', 'admin/save.php', 'admins.html', 'admin/slider.php', 'admin/specializations.php', 'admins.php', 'admin_tool', 'AdminTools', 'admin/uhome.html', 'admin/upload.php', 'admin/userpage.php', 'admin/viewblog.php', 'admin/viewmembers.php', 'admin/voucher.php', 'AdminWeb', 'admin/welcomepage.php', 'admin/welcome.php', 'admloginuser.asp', 'admloginuser.php', 'admon', 'ADMON', 'adm.php', 'affiliate.asp', 'affiliate.php', 'auth', 'auth/login', 'authorize.php', 'autologin', 'banneradmin', 'base/admin', 'bb', 'bbadmin', 'admin/admin.html /bb', 'admin/admin.php /bb', 'admin/index.asp /bb', 'admin/index.html /bb', 'admin/index.php /bb', 'admin/login.asp /bb', 'admin/login.html /bb', 'bigadmin', 'blogindex', 'cadmins', 'ccms', 'ccms/login.php', 'ccp14admin', 'cms', 'cms/admin', 'cmsadmin', 'cms/_admin/logon.php', 'cms/login', 'configuration', 'configure', 'controlpanel.asp', 'controlpanel.html', 'controlpanel.php', 'cPanel', 'cpanel_file', 'cp.asp', 'cp.html', 'cp.php', 'customer_login',
				'database_administration', 'Database_Administration', 'db/admin.php', 'directadmin', 'dir', 'edit.php', 'evmsadmin', 'ezsqliteadmin', 'fileadmin', 'fileadmin.asp', 'fileadmin.html', 'fileadmin.php', 'formslogin', 'forum/admin', 'globes_admin', 'home.asp', 'home.html', 'hpwebjetadmin', 'include/admin.php', 'includes/login.php', 'Indy_admin', 'instadmin', 'interactive/admin.php', 'irc', 'links/login.php', 'LiveUser_Admin', 'login', 'login1', 'login_db', 'loginflat', 'login/login.php', 'login.php', 'redirect', 'logins', 'us', 'logon', 'logo_sysadmin', 'Lotus_Domino_Admin', 'mag/admin', 'maintenance', 'manage_admin.php', 'manager', 'manager/ispmgr', 'manuallogin', 'memberadmin', 'memberadmin.asp', 'memberadmin.php', 'members', 'memlogin', 'meta_login', 'modelsearch', 'modelsearch/admin.html', 'modelsearch/admin.php', 'modelsearch/index.asp', 'modelsearch/index.html', 'index.php', 'modelsearch/login.asp', 'modelsearch/login.html', 'modelsearch/login.php', 'moderator', 'moderator/admin.asp', 'moderator/admin.html', 'moderator/admin.php', 'moderator.asp', 'moderator.html', 'moderator/login.asp', 'moderator/login.html', 'moderator/login.php', 'moderator.php', 'myadmin', 'navSiteAdmin', 'newsadmin', 'nsw/admin/login.php', 'openvpnadmin', 'pages/admin/admin', 'login.php /panel', 'panel', 'administracion/admin.asp', 'administracion/admin.html', 'paneldecontrol', 'panel.php', 'pgadmin', 'phpldapadmin', 'phppgadmin', 'phpSQLiteAdmin', 'platz_login', 'pma', 'power_user', 'project', 'pureadmin', 'radmind', 'rcLogin', 'server', 'ServerAdministrator', 'server_admin_small', 'Server.asp', 'Server.html', 'showlogin', 'simpleLogin', 'site/admin', 'siteadmin', 'siteadmin/index.asp', 'siteadmin/index.php', 'siteadmin/login.asp', 'siteadmin/login.html', 'site_admin/login.php', 'siteadmin/login.php', 'smblogin', 'sql', 'sshadmin', 'ss_vms_admin_sm', 'staradmin', 'sub', 'sysadmins', 'system_administration', 'system', 'user.html', 'utility_login', 'vadmind', 'vmailadmin', 'webadmin/admin.html', 'webadmin/admin.php', 'webadmin.asp', 'webadmin.html', 'webadmin/index.asp', 'webadmin/index.html', 'webadmin/index.php', 'webadmin/login.asp', 'webadmin/login.html', 'webadmin/login.php', 'webadmin.php']

		thread(self, default, url, admin_payloads, thread_count, \
			'admin', status_codes=[200, 401, 403, 201, 202, 204, 206])

	self.save_gather(OUTPUT, 'footprint/interest_files', url, output=self.options['output'])
	return OUTPUT

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, q):
		""" crt.sh search engine

			q 		  : Query for search
		"""
		self.framework = main.framework
		self.q = q
		self._pages = ''
		self._json_page = ''
		self.crt = f"https://crt.sh/?q={q}&output=json"

	def run_crawl(self):
		self.framework.verbose('[CRT] Starting Search...')
		try:
			req = self.framework.request(self.crt)
		except:
			self.framework.debug('[CRT] ConnectionError')
			self.framework.error('CRT is missed!', 'util/crt', 'run_crawl')
			return
		self._pages = req.text
		try:
			self._json_page = req.json()
			if self._json_page == []:
				self._json_page = {}
		except:
			self.framework.error('CRT is missed!')
			self._json_page = []

	@property
	def pages(self):
		return self._pages
	
	@property
	def json_page(self):
		return self._json_page

	@property
	def dns(self):
		return self.framework.page_parse(self.pages).get_dns(self.q)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
        'name': 'SanctionSearch',
        'author': 'Kaushik',
        'version': '0.1',
        'description': 'SanctionSearch is an international list of blacklisted, dangerous\
		personnel and entities.',
        'sources': ('sanctionsearch',),
        'options': (
		('query', None, False, 'Name to search', '-q', 'store', str),
		('id', None, False, 'Id to search', '-i', 'store', int),
		('limit', 15, False, 'Max result count (default=15)', '-l', 'store', int),
        ),
        'examples': ('sanctionsearch -q <QUERY> -l 20',
		'sanctionsearch -i <ID>')
}

NAMESEARCH = False

def module_api(self):
	global NAMESEARCH

	query = self.options['query']
	limit = self.options['limit']
	_id = self.options['id']

	if query is None and _id is None:
		return {}
	elif query is not None and _id is not None:
		_id = None
	elif query is not None:
		NAMESEARCH = True

	run = self.sanctionsearch(query=query, id=_id, limit=limit)
	if NAMESEARCH:
		output_param = query
		run.name_crawl()
		output = {'results': []}
		data = run.data
		for item in data:
			output['results'].append(item)
	else:
		output_param = _id
		run.id_crawl()
		output = run.data
	self.save_gather(output, 'search/sanctionsearch', output_param, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)
	if NAMESEARCH:
		output = output['results']
		for name in output:
			print()
			self.output(name['name'])
			if len(name['address'].strip()) > 0:
				self.output(name['address'])
			self.output(name['link'])
	else:
		if output is not None:
			self.alert_results(output)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Web Crawler',
	'author': 'Saeed',
	'version': '0.5',
	'description': 'Crawl web pages to find links, JS Files, CSS files, Comments\
	 and everything else interesting, supports concurrency.',
	'options': (
		('domain', None, True, 'Domain string', '-d', 'store', str),
		('debug', False, False, 'debug the scraper', '--debug', 'store_true', bool),
		('limit', 1, False, 'Scraper depth level', '-l', 'store', int),
		('thread', 1, False, 'The number of links that open per round', '-t', 'store', int),
	),
	'examples': ('crawler -d <DOMAIN>',
		'crawler -d <DOMAIN> -l 10 -t 3 --output --debug')
}

def module_api(self):
	domain = self.options['domain']
	run = self.web_scrap(domain, self.options['debug'], self.options['limit'], self.options['thread'])
	run.run_crawl()
	output = {'js': run.js, 'cdn': run.cdn,
		 'getlinks': run.query_links, 'exlinks': run.external_links, 
		 'links': run.links, 'css': run.css, 'comments': run.comments, 
		 'emails': run.emails, 'phones': run.phones, 'media': run.media}
	output['usernames'] = run.networks
	self.save_gather(output, 'osint/crawler', domain, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)
	for obj in output:
		self.alert(f"{obj}({len(output[obj])})")
		if output[obj] == []:
			self.output('\t..')
		else:
			for obj_i in output[obj]:
				if obj == 'usernames':
					if output[obj][obj_i] != []:
						self.output(f"\t{obj_i}")
						for user in output[obj][obj_i]:
							self.output(f"\t\t{user}", 'G')
				else:
					self.output(f"\t{obj_i}", 'G')



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

		def __init__(self, query=None, id=None, limit=15):
			""" Sanctionsearch.ofac.treas.gov search engine
					name     : Mame to search
					id       : Id to search
					limit	 : Number of results (only applicable for name search)
			"""
			self.framework = main.framework
			self._query = query
			self._listid = 'ALL'
			self._max = limit
			self._id = id
			self._json = ''
			self.sanctionsearch = 'https://sanctionssearch.ofac.treas.gov/'
			self._rows = []
			self._data = [] if self._query is not None else {}

		def name_crawl(self):
			from bs4 import BeautifulSoup as bs

			self.framework.verbose('Searching sanctionsearch...')
			headers = {
				'Content-Type': 'application/x-www-form-urlencoded',
				}

			try:
				# First request to get viewstate from input tag
				req = self.framework.request(url=self.sanctionsearch)
			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/sanctionsearch', 'name_crawl')
				self.framework.error('Sanctionsearch is missed!', 'util/sanctionsearch', 'name_crawl')
				return

			soup = bs(req.text, 'html.parser')
			viewstate = soup.find('input',
				{'type': 'hidden',
				'name': '__VIEWSTATE',
				'id': '__VIEWSTATE'})['value']

			data = {'__VIEWSTATE': viewstate,
				'ctl00$MainContent$txtLastName': self._query,
				'ctl00$MainContent$btnSearch': 'Search'}
			try:
				req = self.framework.request(
						url=self.sanctionsearch,
						method='POST',
						data=data,
						headers=headers)
			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/sanctionsearch', 'name_crawl')
				self.framework.error('Sanctionsearch is missed!', 'util/sanctionsearch', 'name_crawl')
				return

			soup = bs(req.text, 'html.parser')
			table = soup.find('table', {'id': 'gvSearchResults'})

			self._rows = table.find_all('tr') if table is not None else None

			if self._rows is not None:
				for count, row in enumerate(self._rows):
					if count  >= self._max:
						break

					name = row.find('a').text
					address = row.find_all('td')[1].text
					link = row.find('a')['href']

					self._data.append({
						'name': name,
						'address': address,
						'link': self.sanctionsearch+link
						})

		def id_crawl(self):
			self.framework.verbose('Searching sanctionsearch...')
			url = f"{self.sanctionsearch}Details.aspx?id={self._id}"
			req = self.framework.request(url=url)
			soup = bs(req.text, 'html.parser')

			# DETAILS
			details_table = soup.find('table', {'class': 'MainTable'})

			if details_table is not None:
				details_fields = map(lambda x: x.text, 
					details_table.find_all('tr'))
				details = list(filter(lambda x: len(x) > 0, 
					(''.join(details_fields)).split('\n')))

				self._data['details'] = {}
				for x,y in zip(details, details[1:]):
					if x.endswith(':') and not y.endswith(':'):
						self._data['details'][x] = y

			# IDENTIFICATION
			id_table = soup.find('table', {'id': 'ctl00_MainContent_gvIdentification'})

			if id_table is not None:
				id_th = list(map(lambda x: x.text,
					id_table.find_all('th', {'class': 'borderline'})))
				id_td = list(map(lambda x: x.text,
					id_table.find_all('td')))

				self._data['identification'] = {}
				for i in range(len(id_th)):
					if len(id_td[i].strip()) > 0:
						self._data['identification'][id_th[i]] = id_td[i]

			# AlIASES
			al_table = soup.find('table', {'id': 'ctl00_MainContent_gvAliases'})

			if al_table is not None:
				al_th = list(map(lambda x: x.text,
					al_table.find_all('th', {'class': 'borderline'})))
				al_td = list(map(lambda x: x.text,
					al_table.find_all('td')))

				self._data['aliases'] = {}
				for i in range(len(al_th)):
					if len(al_td[i].strip()) > 0:
						self._data['aliases'][al_th[i]] = al_td[i]

			# ADDRESS
			address_table_div = soup.find('div', {'id': 'ctl00_MainContent_pnlAddress'})

			if address_table_div is not None:
				address_table = address_table_div.find('table')
				address_th = list(map(lambda x: x.text, 
					address_table.find_all('th')))
				address_rows = list(filter(lambda x:len(''.join(x.text).strip()) > 0, 
					address_table.find_all('tr')))[1:]

				self._data['addresses'] = {}
				for i,row in enumerate(address_rows, 1):
					cols = list(map(lambda x: x.text.strip(),
						row.find_all('td')))
					self._data['addresses'][str(i)] = {}
					for j in range(len(cols)):
						if len(cols[j]) > 0:
							self._data['addresses'][str(i)][address_th[j]] = cols[j]

		@property
		def rows(self):
			return self._rows

		@property
		def data(self):
			return self._data



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import re
import os
import json
from time import time
from html import unescape

from maryam.core.basedir import BASEDIR

class main:

	def __init__(self, input_json):
		""" Perform clustering on a set of documents """
		self.framework = main.framework
		self.json = input_json

	def remove_stopwords(self, text):
		stops = open(os.path.join(BASEDIR, 'data', 'stopwords.csv')).read().split(',')
		return [x for x in text if x not in stops]

	def tokenize_and_stem(self, text):
		tokens = re.findall("[A-Z]{2,}(?![a-z])|[A-Z][a-z]+(?=[A-Z])|[\'\w\-]+",text)
		filtered_tokens = []
		for token in tokens:
			if re.search('[a-zA-Z]', token):
				filtered_tokens.append(token.lower())
		return ' '.join(filtered_tokens)

	def punc(self, docs):
		toreturn = []
		for word in self.remove_stopwords(re.findall(r"[\w\-_#]+", docs)):
			if not word.isnumeric():
				toreturn.append(word)
		return ' '.join(toreturn)
	
	def get_frequent_itemsets(self, dataset):
		import pandas as pd
		from mlxtend.frequent_patterns import fpgrowth
		from mlxtend.preprocessing import TransactionEncoder

		te = TransactionEncoder()
		te_ary = te.fit_transform(dataset)
		dataset = pd.DataFrame(te_ary, columns=te.columns_)
		freq_itemsets = fpgrowth(dataset, min_support=0.2, use_colnames=True)
		freq_itemsets = freq_itemsets.sort_values('support', ascending=False)['itemsets']
		freq_itemsets = list(filter(lambda x:len(x)>1, freq_itemsets.values.tolist()))
		return freq_itemsets

	def perform_preprocess(self):
		self.df['d'] = self.df['t'] + ' ' + self.df['d']
		self.df['d'] = self.df['d'].apply(self.tokenize_and_stem).apply(self.punc)

	def perform_clustering(self):
		import pandas as pd

		from sklearn.cluster import KMeans
		from sklearn.feature_extraction.text import TfidfVectorizer

		from kneed import KneeLocator

		self.framework.verbose('Loading Data')
		self.df = pd.DataFrame(self.json['results'])

		self.framework.verbose('Preprocessing')
		self.perform_preprocess()

		vectorizer = TfidfVectorizer(max_features=20, min_df=0.2)
		X = vectorizer.fit_transform(self.df['d'])

		self.framework.verbose('Clustering with KMeans')
		inertias = []
		fitted = []

		for cluster_count in range(1,20):
			km = KMeans(n_clusters=cluster_count)
			km.fit(X)
			inertias.append(km.inertia_)
			fitted.append(km)

		kn = KneeLocator(range(1, len(inertias)+1), inertias, curve='convex', direction='decreasing')
		optimal_k = kn.knee

		best_fit_index = optimal_k-1
		km = fitted[best_fit_index]

		terms = vectorizer.get_feature_names_out()
		order_centroids = km.cluster_centers_.argsort()[:, ::-1]

		summaries = [[] for _ in range(optimal_k)]
		titles = [[] for _ in range(optimal_k)]
		for i in range(optimal_k):
			for index, cluster in enumerate(km.labels_.tolist()):
				if cluster == i:
					summaries[i].append(self.df['d'][index])
					titles[i].append(self.df['t'][index])

		self.framework.verbose('Assigning titles to clusters')
		results = {}
		for index, cluster in enumerate(summaries):
			d = list(map(lambda x: x.split(), cluster))
			self.framework.verbose(f'Assigning title {index+1}')
			probable_titles = self.get_frequent_itemsets(d)
			cluster_title = ' | '.join(list(map(lambda x: ' '.join(list(x)), probable_titles))[:5])
			
			results[cluster_title] = titles[index]
		return results



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import json

class main:

	def __init__(self, q):
		""" to find suggestions for keywords
			example:
			in: 'google'
			out: ['google docs', 'google summer of code', 'google maps', 'google mail', 'google news', ..]

			q 	: Query string
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote
		self._keys_category = []
		self._keys = []
		self._yahoo = \
			'https://search.yahoo.com/sugg/gossip/gossip-us-ura/?f=1&output=sd1&command=<Q>&pq=a&l=3&nresults=30000&b=3&s=1c&callback=<b>'
		self._google = 'https://www.google.com/complete/search?q=<Q>&cp=&client=psy-ab&xssi=t&gs_ri=gws-wiz&hl=&authuser=0&psi='
		self._bing = 'https://www.bing.com/AS/Suggestions?pt=&mkt=de-de&qry=<Q>&cp=0&css=0&cvid=1'
		self._millionshort = 'https://millionshort.com/api/suggestions?q=<Q>'
		self._zapmeta = 'https://www.zapmeta.com/suggest?q=<Q>'
		self._searx = 'https://searx.be/autocompleter?q=<Q>'
		self._peekier = {'url': 'https://search.peekier.com/suggestions', 'payload': {'q': '<Q>', 'region': ''}, 'method': 'POST'}
		self._gigablast = 'http://gigablast.com/qs?rwp=0&lang=en&q=<Q>'

	def run_crawl(self):
		keys = {}
		for source in ('yahoo', 'google', 'bing', 'millionshort',\
						'zapmeta', 'searx', 'peekier', 'gigablast'):
			attr = getattr(self, '_'+source)
			if isinstance(attr, dict):
				url = attr['url']
				method = 'POST'
				data = attr.get('payload', {'q':'<Q>'})
				data['q'] = self.q
			else:
				url = attr.replace('<Q>', self.q)
				method = 'GET'
				data = {}
			try:
				req = self.framework.request(url, method=method, data=data)
			except Exception as e:
				self.framework.error('ConnectionError', 'util/keywords', 'run_crawl')
				self.framework.print_exception()
			keys[source] = req

		keys['yahoo'] = [x['k'] for x in keys['yahoo'].json().get('r', [])]
		try:
			google = json.loads(f"{keys['google'].text[5:]}")[0]
			keys['google'] = [re.sub(r"<b>|<\\/b>|</b>", '', x[0]) for x in google]
		except:
			keys['google'] = []
		keys['bing'] = re.findall(r'<span class="sa_tm_text">([^<]+)</span>', re.sub(r'<.?strong>', '', keys['bing'].text))
		keys['zapmeta'] = [x[0] for x in keys['zapmeta'].json()] if hasattr(keys['zapmeta'], 'json') else []
		keys['millionshort'] = keys['millionshort'].json().get('suggestions', []) if hasattr(keys['millionshort'], 'json') else []
		try:
			keys['searx'] = keys['searx'].json()[self.q]
		except:
			keys['searx'] = []
		keys['peekier'] = keys['peekier'].json()['results']
		keys['gigablast'] = re.findall(r'" >([^\n]+?)</td', keys['gigablast'].text)
		self._keys_category = keys
		for s in keys:
			for i in keys[s]:
				self._keys.append(i)

	@property
	def keys_category(self):
		return self._keys_category

	@property
	def keys(self):
		return list(set(self._keys))



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, q, limit):
		"""
		photon uses osm data for locations and reverse geolocation
		q       :   query
		limit   :   max result count
		"""

		self.framework = main.framework
		self.q = q.split('_')[0]
		self.lat = q.split('_')[1]
		self.lon = q.split('_')[2]
		self._pages = ''
		self._json = {}
		self.ph = 'https://photon.komoot.io/'

	def run_crawl(self):
		urls = [f"{self.ph}api/?",f"{self.ph}reverse?"]
		if self.q and self.lat and self.lon:
			if self.q == 'q':
				url = urls[1]
				payloads = {'lat': self.lat, 'lon': self.lon}
			else:
				url = urls[0]
				payloads = {'q': self.q, 'lat': self.lat, 'lon': self.lon}
		else:
			url = urls[0]
			payloads = {'q': self.q}
		self.framework.verbose("[PHOTON]Searching in photon.komoot.io...")
		try:
			req = self.framework.request(
				url = url,
				params = payloads
			)
			result = req.text
			result_json = req.json()
		except:
			self.framework.error('ConnectionError', 'util/photon', 'run_crawl')
		else:
			if 'message' in req.text:
				self.framework.verbose(f"[PHOTON] Search ended with the message: {req.json()['message'].split(',')[0]}")
		self._pages = result
		self._json = result_json

	@property
	def pages(self):
		return self._pages

	@property
	def json(self):
		return self._json


"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import random

class main:

		def __init__(self, num):
			""" numverify search engine
					num		: number to search
					country : country code
					
			"""
			self.framework = main.framework
			self.num = num
			self._json = ''
			self.numverify = 'http://apilayer.net/api/validate'
			self.keys = [
				'3489ab1ab092628f8b2a63b3d94b75f3',
				'ea52c77e9877dd4fdb83bdec925d3eeb',
				'e6f3944b3724f836871a4f9dcc546eb6',
				'3796398e6c96912c29a56ec0177b8d6b',
				'95f0bc6561937986c3386b1ffcd36ae9',
				'd49245f8853efbff4dfd245387531727',
				'62552e99ed3cb92159be4153242b7eea',
				'1322a4a256acd589e2e3ce73e65fdd3f',
				'3fcf94548f2177e781eff270d43790f6',
				'7778341260206c06fac4a2727a27955f',
				'a0bfc16589d513fc472832935c9c6528',
				'3618d844c1b6802106a6309c92c71ac2',
				'376bbc8ef5ee4a9d8f6a81a7ddd9f4cd',
				'd4e8383a4f2efec6e523faad8797c4d1',
				'0a45e3f8273a7f929f4ec237e4f24d10',
				'64edafd9cc5d839d8d9bac9ac0837851',
				'5165b0fec39bb03e42f43d4cc5513dcb',
				'e2a851b9b76a579c0fef4304969546e7',
				'9a39c3058615ec330c7223a596f62934',
				'2a06b63cb88a13ebeea7e5f83f01f8d7',
				'1ed52249226585cdb176926e34d75aa5',
				'8a6cb00cac3c904dd1c82b29ab5061d0',
				'fe11bd5cfea457f9fda5ed3fecc8fc54'
			]

		def run_crawl(self):
			self.framework.verbose('Searching...')
			try:
				payload = { 
						'access_key': random.choice(self.keys),
						'number': self.num,
						'format': 1
				}
				req = self.framework.request(
						url=self.numverify,
						params=payload)
				self._json = req.json()
			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/numverify', 'run_crawl')
				self.framework.error('Numverify is missed!', 'util/numverify', 'run_crawl')
				return

		@property
		def json(self):
			return self._json



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import json

class main:

		def __init__(self, q):
			""" Google Dictionary search engine
					q     : Query to search
			"""
			self.framework = main.framework
			self.q = q
			self._json = ''
			self.dictionary = 'https://api.dictionaryapi.dev/api/v2/entries/en_US/'

		def run_crawl(self):
			self.q = self.framework.urlib(self.q).quote
			self.url = f"{self.dictionary}{self.q}"

			self.framework.verbose('Searching Dictionary...')
			try:
				req = self.framework.request(url=self.url)
				self._json = req.json()
			except:
				self.framework.error('ConnectionError.', 'util/dictionary', 'run_crawl')
				self.framework.error('Dictionary is missed!', 'util/dictionary', 'run_crawl')

		@property
		def json(self):
			return self._json

		@property
		def raw(self):
			return json.dumps(self._json)

		@property
		def definitions(self):
			definition_list = []

			if not (type(self._json) == dict and \
					self._json.get('title') == 'No Definitions Found'):
				for context in self._json:
					meanings = context['meanings']
					if len(meanings) > 0:
						for meaning in meanings:
							partOfSpeech = meaning['partOfSpeech']
							definition_sections = meaning['definitions']

							for section in definition_sections:
								definition_list.append({
									'partOfSpeech': partOfSpeech.title(),
									'definition': section['definition']
									})
			return definition_list
